#!/usr/bin/env python
"""Tests of the bitvector module.
"""
from cogent.util.unit_test import TestCase, main
from cogent.core.bitvector import is_nonzero_string_char, is_nonzero_char, \
    seq_to_bitstring, is_nonzero_string_int, is_nonzero_int, seq_to_bitlist,\
    num_to_bitstring, bitcount, Bitvector, MutableBitvector, \
    ImmutableBitvector, VectorFromCases, VectorFromMatches, VectorFromRuns, \
    VectorFromSpans, VectorFromPositions, PackedBases, \
    LongBitvector, ShortBitvector
import re

__author__ = "Jeremy Widmann"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jeremy Widmann", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Jeremy Widmann"
__email__ = "jeremy.widmann@colorado.edu"
__status__ = "Production"

class bitvectorTests(TestCase):
    """Tests of top-level functions."""

    def test_is_nonzero_string_char(self):
        """is_nonzero_string_char should return '1' for anything but '0', ''"""
        self.assertEqual(is_nonzero_string_char('0'), '0')
        self.assertEqual(is_nonzero_string_char(''), '0')
        for char in "QWERTYUIOPASDFGHJKL:ZXCGHJMK<L>?{|!@#$%^&*()12345678":
            self.assertEqual(is_nonzero_string_char(char), '1')

    def test_is_nonzero_char(self):
        """is_nonzero_char should return '0' for any False item or '0'"""
        zero = ['', 0, '0', [], {}, None, 0L, 0.0, False]
        for z in zero:
            self.assertEqual(is_nonzero_char(z), '0')
        nonzero = ['z', '1', '00', ' ', 1, -1, 1e-30, [''], {'':None}, True]
        for n in nonzero:
            self.assertEqual(is_nonzero_char(n), '1')

    def test_seq_to_bitstring(self):
        """seq_to_bitstring should provide expected results"""
        zero = ['', 0, '0', [], {}, None, 0L, 0.0, False]
        self.assertEqual(seq_to_bitstring(zero), '0'*9)
        nonzero = ['z', '1', '00', ' ', 1, -1, 1e-30, [''], {'':None}, True]
        self.assertEqual(seq_to_bitstring(nonzero), '1'*10)
        self.assertEqual(seq_to_bitstring(''), '')
        self.assertEqual(seq_to_bitstring('305'), '101')
        self.assertEqual(seq_to_bitstring(''), '')

    def test_is_nonzero_string_int(self):
        """is_nonzero_string_int should return 1 for anything but '0', ''"""
        self.assertEqual(is_nonzero_string_int('0'), 0)
        self.assertEqual(is_nonzero_string_int(''), 0)
        for char in "QWERTYUIOPASDFGHJKL:ZXCGHJMK<L>?{|!@#$%^&*()12345678":
            self.assertEqual(is_nonzero_string_int(char), 1)

    def test_is_nonzero_int(self):
        """is_nonzero_int should return 0 for any False item or '0'"""
        zero = ['', 0, '0', [], {}, None, 0L, 0.0, False]
        for z in zero:
            self.assertEqual(is_nonzero_int(z), 0)
        nonzero = ['z', '1', '00', ' ', 1, -1, 1e-30, [''], {'':None}, True]
        for n in nonzero:
            self.assertEqual(is_nonzero_int(n), 1)

    def test_seq_to_bitlist(self):
        """seq_to_bitlist should provide expected results"""
        zero = ['', 0, '0', [], {}, None, 0L, 0.0, False]
        self.assertEqual(seq_to_bitlist(zero), [0]*9)
        nonzero = ['z', '1', '00', ' ', 1, -1, 1e-30, [''], {'':None}, True]
        self.assertEqual(seq_to_bitlist(nonzero), [1]*10)
        self.assertEqual(seq_to_bitlist(''), [])
        self.assertEqual(seq_to_bitlist('305'), [1,0,1])
        self.assertEqual(seq_to_bitlist(''), [])

    def test_number_to_bitstring(self):
        """number_to_bitstring should provide expected results"""
        numbers = [0, 1, 2, 7, 8, 1024, 814715L]
        for n in numbers:
            self.assertEqual(num_to_bitstring(n, 0), '')

        single_results = list('0101001')
        for exp, num in zip(single_results, numbers):
            self.assertEqual(num_to_bitstring(num, 1), exp)

        three_results = ['000','001','010','111','000','000','011']
        for exp, num in zip(three_results, numbers):
            self.assertEqual(num_to_bitstring(num, 3), exp)
        
        #should pad or truncate to the correct length
        self.assertEqual(num_to_bitstring(814715, 20),'11000110111001111011')
        self.assertEqual(num_to_bitstring(814715, 10),'1001111011')
        self.assertEqual(num_to_bitstring(8, 10),'0000001000')

    def test_bitcount(self):
        """bitcount should provide expected results"""
        numbers = [0, 1, 2, 7, 8, 1024, 814715L]

        twenty_results = [0, 1, 1, 3, 1, 1, 13]
        for exp, num in zip(twenty_results, numbers):
            self.assertEqual(bitcount(num, 20), exp)
            self.assertEqual(bitcount(num, 20, 1), exp)
            self.assertEqual(bitcount(num, 20, 0), 20 - exp)

        three_results = [0,1,1,3,0,0,2]
        for exp, num in zip(three_results, numbers):
            self.assertEqual(bitcount(num, 3), exp)
            self.assertEqual(bitcount(num, 3, 1), exp)
            self.assertEqual(bitcount(num, 3, 0), 3 - exp)

        for num in numbers:
            self.assertEqual(bitcount(num, 0), 0)
            self.assertEqual(bitcount(num, 0, 0), 0)
            self.assertEqual(bitcount(num, 0, 1), 0)
        
class BitvectorTests(TestCase):
    """Tests of the (immutable) Bitvector class."""

    def setUp(self):
        """Define a few standard strings and vectors."""
        self.strings = ['', '0', '1', '00', '01', '10', '11']
        self.vectors = map(Bitvector, self.strings)

    def test_init(self):
        """Bitvector init should give expected results."""
        self.assertEqual(Bitvector(), 0)
        self.assertEqual(Bitvector('1001'), 9)
        self.assertEqual(Bitvector(['1','0','0','0']), 8)
        self.assertEqual(Bitvector([]), 0)
        #if passing in non-sequence, must specify length
        self.assertRaises(TypeError, Bitvector, 1024)
        self.assertEqual(Bitvector(1024, 10), 1024)
        bv = Bitvector(10, 3)
        self.assertEqual(bv, 10)
        self.assertEqual(len(bv), 3)
        self.assertEqual(len(Bitvector('1'*1000)), 1000)
        #check that initializing a bv from itself preserves length
        bv2 = Bitvector(bv)
        self.assertEqual(bv2, 10)
        self.assertEqual(len(bv2), 3)

    def test_len(self):
        """Bitvector len should match initialized length"""
        self.assertEqual(len(Bitvector()), 0)
        self.assertEqual(len(Bitvector('010')), 3)
        self.assertEqual(len(Bitvector(1024, 5)), 5)
        self.assertEqual(len(Bitvector(1024, 0)), 0)
        self.assertEqual(len(Bitvector('1'*1000)), 1000)

    def test_str(self):
        """Bitvector str should match expected results"""
        vecs = [Bitvector(i, 0) for i in [0, 1, 2, 7, 8, 1024, 814715L]]
        for v in vecs:
            self.assertEqual(str(v), '')

        vecs = [Bitvector(i, 1) for i in [0, 1, 2, 7, 8, 1024, 814715L,'1'*50]]
        single_results = list('01010011')
        for exp, vec in zip(single_results, vecs):
            self.assertEqual(str(vec), exp)

        vecs = [Bitvector(i, 3) for i in [0, 1, 2, 7, 8, 1024, 814715L,'1'*50]]
        three_results = ['000','001','010','111','000','000','011','111']
        for exp, vec in zip(three_results, vecs):
            self.assertEqual(str(vec), exp)
        
        #should pad or truncate to the correct length
        self.assertEqual(str(Bitvector(814715, 20)),'11000110111001111011')
        self.assertEqual(str(Bitvector(814715, 10)),'1001111011')
        self.assertEqual(str(Bitvector(8, 10)),'0000001000')
        self.assertEqual(str(Bitvector('1'*50)), '1'*50)

    def test_or(self):
        """Bitvector A|B should return 1 for each position that is 1 in A or B"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' or x
                    ['', '0', '1', '0', '0', '1', '1'],         #'0' or x
                    ['', '1', '1', '1', '1', '1', '1'],         #'1' or x
                    ['', '0', '1', '00', '01', '10', '11'],     #'00' or x
                    ['', '0', '1', '01', '01', '11', '11'],     #'01' or x
                    ['', '1', '1', '10', '11', '10', '11'],     #'10' or x
                    ['', '1', '1', '11', '11', '11', '11'],     #'11' or x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first | second),
                                    results[first_pos][second_pos])
        #test chaining
        expected = Bitvector('1110')
        observed = Bitvector('1000') | Bitvector('0100') | Bitvector('0110')
        self.assertEqual(observed, expected)

        #test long
        self.assertEqual(Bitvector('10'*50) | Bitvector('01'*50), \
            Bitvector('11'*50))
        
    def test_and(self):
        """Bitvector A&B should return 0 for each position that is 0 in A and B"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' and x
                    ['', '0', '0', '0', '0', '0', '0'],         #'0' and x
                    ['', '0', '1', '0', '0', '1', '1'],         #'1' and x
                    ['', '0', '0', '00', '00', '00', '00'],     #'00' and x
                    ['', '0', '0', '00', '01', '00', '01'],     #'01' and x
                    ['', '0', '1', '00', '00', '10', '10'],     #'10' and x
                    ['', '0', '1', '00', '01', '10', '11'],     #'11' and x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first & second), 
                                    results[first_pos][second_pos])
        #test chaining
        expected = Bitvector('0110')
        observed = Bitvector('1110') & Bitvector('1111') & Bitvector('0111')
        self.assertEqual(observed, expected)

        #test long
        self.assertEqual(Bitvector('10'*50) & Bitvector('11'*50), \
            Bitvector('10'*50))
        
    def test_xor(self):
        """Bitvector A^B should return 0 for each identical position in A and B"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' xor x
                    ['', '0', '1', '0', '0', '1', '1'],         #'0' xor x
                    ['', '1', '0', '1', '1', '0', '0'],         #'1' xor x
                    ['', '0', '1', '00', '01', '10', '11'],     #'00' xor x
                    ['', '0', '1', '01', '00', '11', '10'],     #'01' xor x
                    ['', '1', '0', '10', '11', '00', '01'],     #'10' xor x
                    ['', '1', '0', '11', '10', '01', '00'],     #'11' xor x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first ^ second),
                                    results[first_pos][second_pos])

        #test chaining
        expected = Bitvector('0110')
        observed = Bitvector('1111') ^ Bitvector('0110') ^ Bitvector('1111')
 
        #test long
        self.assertEqual(Bitvector('11'*50) ^ Bitvector('01'*50), \
            Bitvector('10'*50))
        
    def test_invert(self):
        """Bitvector ~A should return a vector exchanging 1's for 0's"""
        results = map(Bitvector, ['', '1', '0', '11', '10', '01', '00'])
        for data, result in zip(self.vectors, results):
            self.assertEqual(~data, result)
            
            if len(data):
                self.assertNotEqual(data, result)
            else:
                self.assertEqual(data, result)

            #test chaining
            self.assertEqual(~~data, data) #inverting twice should give original
            self.assertEqual(~~~data, ~data)

            #test long
            self.assertEqual(~Bitvector('10'*50), Bitvector('01'*50))
            self.assertEqual(str(~Bitvector('10'*50)), str(Bitvector('01'*50)))
   
    def test_getitem(self):
        """Bitvector getitem should return states at specified position(s)"""

        vec_strings = ['', '0', '1', '10', '10001101', '101'*50]
        vecs = map(Bitvector, vec_strings)
        for vec_string, vec in zip(vec_strings, vecs):
            for char, item in zip(vec_string, vec):
                self.assertEqual(char, str(item))
        #test some 2- and 3-item slices as well
        vec = Bitvector('1001000101001')
        self.assertEqual(vec[3:7], Bitvector('1000'))
        self.assertEqual(vec[:4], Bitvector('1001'))
        self.assertEqual(vec[7:], Bitvector('101001'))
        self.assertEqual(vec[1:11:2], Bitvector('01011'))
        
    def test_bitcount(self):
        """Bitvector bitcount should correctly count 1's or 0's"""
        vec_strings = ['', '0', '1', '10', '10001101', '101'*50]
        vecs = map(Bitvector, vec_strings)
        one_counts = [0, 0, 1, 1, 4, 100]
        zero_counts = [0, 1, 0, 1, 4, 50]
        for v, o, z in zip(vecs, one_counts, zero_counts):
            self.assertEqual(v.bitcount(), o)
            self.assertEqual(v.bitcount(1), o)
            self.assertEqual(v.bitcount(0), z)

    def test_repr(self):
        """Bitvector repr should look like a normal object"""
        v = Bitvector(3, 10)
        v_id = str(hex(id(v)))
        expected = '<cogent.core.bitvector.ShortBitvector object at'
        self.assertTrue(`v`.startswith(expected))

    def test_freeze(self):
        """Bitvector freeze should return same object"""
        v = Bitvector()
        self.assertSameObj(v.freeze(), v)

    def test_thaw(self):
        """Bitvector thaw should return mutable bitvector with same data"""
        b = Bitvector('111')
        c = b.thaw()
        self.assertEqual(c, Bitvector('111'))
        c[1] = 0
        self.assertEqual(c, Bitvector('101'))
        self.assertNotEqual(Bitvector('111'), Bitvector('101'))

    def test_stateChanges(self):
        """Bitvector stateChanges should return indices where state changes"""
        vec_strings = ['', '0', '1', '10', '111', '10001101', '1111100000'*5]
        vecs = map(Bitvector, vec_strings)
        results = [
            [],
            [0,1],
            [0,1],
            [0,1,2],
            [0,3],
            [0,1,4,6,7,8],
            [0,5,10,15,20,25,30,35,40,45,50],
        ]
        for vec, res in zip(vecs, results):
            self.assertEqual(vec.stateChanges(), res)
    
    def test_divideSequence(self):
        """Bitvector divideSequence should cut sequence at state changes"""
        vec_strings = ['', '0', '1', '10', '111', '10001101', '1111100000'*5,
            '0000011111']
        vecs = map(Bitvector, vec_strings)
        seq = 'abc'*30
        results_none = [
            ([], 0),
            (['a'], 0),
            (['a'], 1),
            (['a','b'], 1),
            (['abc'], 1),
            (['a','bca','bc','a','b'], 1),
            (['abcab','cabca','bcabc','abcab','cabca','bcabc','abcab','cabca',
            'bcabc', 'abcab'], 1),
            (['abcab','cabca'], 0),
        ]
        for vec, res in zip(vecs, results_none):
            self.assertEqual(vec.divideSequence(seq), res)

        short_seq = [1,2,3]
        results_short = [
            ([], 0),
            ([[1]],0),
            ([[1]],1),
            ([[1],[2]],1),
            ([[1,2,3]],1),
            ([[1],[2,3]],1),
            ([[1,2,3]],1),
            ([[1,2,3]],0),
        ]
        for vec, res in zip(vecs, results_short):
            self.assertEqual(vec.divideSequence(short_seq), res)
         
        results_zero = [
            ([], 0),
            (['a'], 0),
            ([], 1),
            (['b'], 1),
            ([], 1),
            (['bca','a'], 1),
            (['cabca','abcab','bcabc','cabca', 'abcab'], 1),
            (['abcab'], 0),
        ]
        for vec, res in zip(vecs, results_zero):
            self.assertEqual(vec.divideSequence(seq, 0), res)

        results_one = [
            ([], 0),
            ([], 0),
            (['a'], 1),
            (['a'], 1),
            (['abc'], 1),
            (['a','bc','b'], 1),
            (['abcab','bcabc','cabca','abcab','bcabc'], 1),
            (['cabca'], 0),
        ]
        for vec, res in zip(vecs, results_none):
            self.assertEqual(vec.divideSequence(seq), res)

class MutableBitvectorTests(TestCase):
    """Tests of the MutableBitvector class."""

    def setUp(self):
        """Define a few standard strings and vectors."""
        self.strings = ['', '0', '1', '00', '01', '10', '11']
        self.vectors = map(MutableBitvector, self.strings)

    def test_init(self):
        """MutableBitvector init should give expected results."""
        self.assertEqual(MutableBitvector(), 0)
        self.assertEqual(MutableBitvector('1001'), 9)
        self.assertEqual(MutableBitvector(['1','0','0','0']), 8)
        self.assertEqual(MutableBitvector([]), 0)
        #if passing in non-sequence, must specify length
        self.assertRaises(TypeError, MutableBitvector, 1024)
        self.assertEqual(MutableBitvector(1024, 10), 1024)
        bv = MutableBitvector(10, 5)
        self.assertEqual(bv, 10)
        self.assertEqual(len(bv), 5)
        self.assertEqual(len(MutableBitvector('1'*1000)), 1000)
        #check that initializing a bv from itself preserves length
        bv2 = MutableBitvector(bv)
        self.assertEqual(bv2, 10)
        self.assertEqual(len(bv2), 5)

    def test_len(self):
        """MutableBitvector len should match initialized length"""
        self.assertEqual(len(MutableBitvector()), 0)
        self.assertEqual(len(MutableBitvector('010')), 3)
        self.assertEqual(len(MutableBitvector(1024, 5)), 5)
        self.assertEqual(len(MutableBitvector(1024, 0)), 0)
        self.assertEqual(len(MutableBitvector('1'*1000)), 1000)

    def test_str(self):
        """MutableBitvector str should match expected results"""
        vecs = [MutableBitvector(i, 0) for i in [0, 1, 2, 7, 8, 1024, 814715L]]
        for v in vecs:
            self.assertEqual(str(v), '')

        vecs = [MutableBitvector(i, 1) for i in [0, 1, 2, 7, 8, 1024, 814715L,'1'*50]]
        single_results = list('01010011')
        for exp, vec in zip(single_results, vecs):
            self.assertEqual(str(vec), exp)

        vecs = [MutableBitvector(i, 3) for i in [0, 1, 2, 7, 8, 1024, 814715L,'1'*50]]
        three_results = ['000','001','010','111','000','000','011','111']
        for exp, vec in zip(three_results, vecs):
            self.assertEqual(str(vec), exp)
        
        #should pad or truncate to the correct length
        self.assertEqual(str(MutableBitvector(814715, 20)),'11000110111001111011')
        self.assertEqual(str(MutableBitvector(814715, 10)),'1001111011')
        self.assertEqual(str(MutableBitvector(8, 10)),'0000001000')
        self.assertEqual(str(MutableBitvector('1'*50)), '1'*50)

    def test_or(self):
        """MutableBitvector A|B should return 1 at 1 positions in A or B"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' or x
                    ['', '0', '1', '0', '0', '1', '1'],         #'0' or x
                    ['', '1', '1', '1', '1', '1', '1'],         #'1' or x
                    ['', '0', '1', '00', '01', '10', '11'],     #'00' or x
                    ['', '0', '1', '01', '01', '11', '11'],     #'01' or x
                    ['', '1', '1', '10', '11', '10', '11'],     #'10' or x
                    ['', '1', '1', '11', '11', '11', '11'],     #'11' or x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first | second),
                                    results[first_pos][second_pos])
        #test chaining
        expected = MutableBitvector('1110')
        observed = MutableBitvector('1000') | \
                   MutableBitvector('0100') | \
                   MutableBitvector('0110')
        self.assertEqual(observed, expected)

        #test long
        self.assertEqual(MutableBitvector('10'*50) | MutableBitvector('01'*50), \
            MutableBitvector('11'*50))
        
    def test_and(self):
        """MutableBitvector A&B should return 0 for each position 0 in both"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' and x
                    ['', '0', '0', '0', '0', '0', '0'],         #'0' and x
                    ['', '0', '1', '0', '0', '1', '1'],         #'1' and x
                    ['', '0', '0', '00', '00', '00', '00'],     #'00' and x
                    ['', '0', '0', '00', '01', '00', '01'],     #'01' and x
                    ['', '0', '1', '00', '00', '10', '10'],     #'10' and x
                    ['', '0', '1', '00', '01', '10', '11'],     #'11' and x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first & second),
                                    results[first_pos][second_pos])
        #test chaining
        expected = MutableBitvector('0110')
        observed = MutableBitvector('1110') & \
                   MutableBitvector('1111') & \
                   MutableBitvector('0111')
        self.assertEqual(observed, expected)
        
        #test long
        self.assertEqual(MutableBitvector('10'*50) & MutableBitvector('11'*50), \
            MutableBitvector('10'*50))
        
    def test_xor(self):
        """MutableBitvector A^B should return 0 at identical positions in A, B"""

        results = [
                    ['', '', '', '', '', '', ''],               #'' xor x
                    ['', '0', '1', '0', '0', '1', '1'],         #'0' xor x
                    ['', '1', '0', '1', '1', '0', '0'],         #'1' xor x
                    ['', '0', '1', '00', '01', '10', '11'],     #'00' xor x
                    ['', '0', '1', '01', '00', '11', '10'],     #'01' xor x
                    ['', '1', '0', '10', '11', '00', '01'],     #'10' xor x
                    ['', '1', '0', '11', '10', '01', '00'],     #'11' xor x
                ]
        vectors = self.vectors

        for first_pos, first in enumerate(vectors):
            for second_pos, second in enumerate(vectors):
                self.assertEqual(   str(first ^ second),
                                    results[first_pos][second_pos])

        #test chaining
        expected = MutableBitvector('0110')
        observed = MutableBitvector('1111') ^ \
                   MutableBitvector('0110') ^ \
                   MutableBitvector('1111')
        self.assertEqual(observed, expected)
 
        #test long
        self.assertEqual(MutableBitvector('11'*50) ^ MutableBitvector('01'*50), \
            MutableBitvector('10'*50))
        
    def test_invert(self):
        """MutableBitvector ~A should return a vector exchanging 1's for 0's"""
        results = map(MutableBitvector, ['', '1', '0', '11', '10', '01', '00'])
        for data, result in zip(self.vectors, results):
            self.assertEqual(~data, result)
            
            if len(data):
                self.assertNotEqual(data, result)
            else:
                self.assertEqual(data, result)

            #test chaining
            self.assertEqual(~~data, data) #inverting twice should give original
            self.assertEqual(~~~data, ~data)

            #test long
            self.assertEqual(~MutableBitvector('10'*50), MutableBitvector('01'*50))
            self.assertEqual(str(~MutableBitvector('10'*50)), str(MutableBitvector('01'*50)))
   
    def test_getitem(self):
        """MutableBitvector getitem should return states at specified position(s)"""

        vec_strings = ['', '0', '1', '10', '10001101', '101'*50]
        vecs = map(MutableBitvector, vec_strings)
        for vec_string, vec in zip(vec_strings, vecs):
            for char, item in zip(vec_string, vec):
                self.assertEqual(char, str(item))
        #test some 2- and 3-item slices as well
        vec = MutableBitvector('1001000101001')
        self.assertEqual(vec[3:7], MutableBitvector('1000'))
        self.assertEqual(vec[:4], MutableBitvector('1001'))
        self.assertEqual(vec[7:], MutableBitvector('101001'))
        self.assertEqual(vec[1:11:2], MutableBitvector('01011'))
       

    def test_setitem(self):
        """MutableBitvector setitem should change positions correctly"""
        self.assertRaises(IndexError, MutableBitvector().__setitem__, 1, 1)
        vec_strings = ['0', '1', '10', '10001101', '101'*50]
        vecs = map(MutableBitvector, vec_strings)
        results_1 = ['1', '1', '10', '10001101', '101'*50]
        for vec, res  in zip(vecs, results_1):
            vec[0] = 1
            self.assertEqual(vec, Bitvector(res))

        results_0 = ['0', '0', '10', '10001100', '101'*49+'100']
        for vec, res  in zip(vecs, results_0):
            vec[-1] = 0
            self.assertEqual(vec, Bitvector(res))
            
        vec = MutableBitvector('1001000101001')
        vec[3:7] = '1111'
        self.assertEqual(vec,  Bitvector('1001111101001'))
        vec[:4] = '0101'
        self.assertEqual(vec,  Bitvector('0101111101001'))
        vec[7:] = '000111'
        self.assertEqual(vec,  Bitvector('0101111000111'))
        vec[1:11:2] = '11011'
        self.assertEqual(vec,  Bitvector('0101101101111'))

    def test_bitcount(self):
        """MutableBitvector bitcount should correctly count 1's or 0's"""
        vec_strings = ['', '0', '1', '10', '10001101', '101'*50]
        vecs = map(MutableBitvector, vec_strings)
        one_counts = [0, 0, 1, 1, 4, 100]
        zero_counts = [0, 1, 0, 1, 4, 50]
        for v, o, z in zip(vecs, one_counts, zero_counts):
            self.assertEqual(v.bitcount(), o)
            self.assertEqual(v.bitcount(1), o)
            self.assertEqual(v.bitcount(0), z)

    def test_repr(self):
        """MutableBitvector repr should look like a normal object"""
        v = MutableBitvector(3, 10)
        v_id = str(hex(id(v)))
        expected = '<cogent.core.bitvector.MutableBitvector object at'
        self.assertTrue(`v`.startswith(expected))

    def test_thaw(self):
        """MutableBitvector thaw should return same object"""
        v = MutableBitvector()
        self.assertSameObj(v.thaw(), v)

    def test_freeze(self):
        """MutableBitvector freeze should return immutable bv with same data"""
        b = MutableBitvector('111')
        b[1] = 0
        c = b.freeze()
        self.assertSameObj(c, b._handler)
        self.assertEqual(c, Bitvector('101'))
        try:
            c[1] = 1
        except TypeError:
            pass
        else:
            raise AssertionError, \
                "MutableBitvector.freeze() returned mutable object."

    def test_stateChanges(self):
        """MutableBitvector stateChanges should return indices where state changes"""
        vec_strings = ['', '0', '1', '10', '111', '10001101', '1111100000'*5]
        vecs = map(MutableBitvector, vec_strings)
        results = [
            [],
            [0,1],
            [0,1],
            [0,1,2],
            [0,3],
            [0,1,4,6,7,8],
            [0,5,10,15,20,25,30,35,40,45,50],
        ]
        for vec, res in zip(vecs, results):
            self.assertEqual(vec.stateChanges(), res)
    
    def test_divideSequence(self):
        """MutableBitvector divideSequence should cut sequence at state changes"""
        vec_strings = ['', '0', '1', '10', '111', '10001101', '1111100000'*5,
            '0000011111']
        vecs = map(MutableBitvector, vec_strings)
        seq = 'abc'*30
        results_none = [
            ([], 0),
            (['a'], 0),
            (['a'], 1),
            (['a','b'], 1),
            (['abc'], 1),
            (['a','bca','bc','a','b'], 1),
            (['abcab','cabca','bcabc','abcab','cabca','bcabc','abcab','cabca',
            'bcabc', 'abcab'], 1),
            (['abcab','cabca'], 0),
        ]
        for vec, res in zip(vecs, results_none):
            self.assertEqual(vec.divideSequence(seq), res)

        short_seq = [1,2,3]
        results_short = [
            ([], 0),
            ([[1]],0),
            ([[1]],1),
            ([[1],[2]],1),
            ([[1,2,3]],1),
            ([[1],[2,3]],1),
            ([[1,2,3]],1),
            ([[1,2,3]],0),
        ]
        for vec, res in zip(vecs, results_short):
            self.assertEqual(vec.divideSequence(short_seq), res)
         
        results_zero = [
            ([], 0),
            (['a'], 0),
            ([], 1),
            (['b'], 1),
            ([], 1),
            (['bca','a'], 1),
            (['cabca','abcab','bcabc','cabca', 'abcab'], 1),
            (['abcab'], 0),
        ]
        for vec, res in zip(vecs, results_zero):
            self.assertEqual(vec.divideSequence(seq, 0), res)

        results_one = [
            ([], 0),
            ([], 0),
            (['a'], 1),
            (['a'], 1),
            (['abc'], 1),
            (['a','bc','b'], 1),
            (['abcab','bcabc','cabca','abcab','bcabc'], 1),
            (['cabca'], 0),
        ]
        for vec, res in zip(vecs, results_none):
            self.assertEqual(vec.divideSequence(seq), res)

class BitvectorClassTests(TestCase):
    """Bit operations should work correctly for different bitvector types."""
    def setUp(self):
        """Define a few standard vectors"""
        self.s = ShortBitvector('00000000000000101111000000001011000')
        self.l = LongBitvector('11111111111111111111111111111111111')
        self.tiny = ShortBitvector('1')
        self.huge = LongBitvector('1'*1000)

    def test_and(self):
        """Bitwise and should work between short and long bitvectors"""
        s, l, tiny, huge = self.s, self.l, self.tiny, self.huge
        self.assertTrue(isinstance(l, long))
        self.assertEqual(l & s, s)
        self.assertEqual(s & l, s)
        self.assertEqual(huge & tiny, tiny)
        self.assertEqual(tiny & huge, tiny)

    def test_or(self):
        """Bitwise or should work between short and long bitvectors"""
        s, l, tiny, huge = self.s, self.l, self.tiny, self.huge
        self.assertEqual(l | s, l)
        self.assertEqual(s | l, l)
        self.assertEqual(huge | tiny, tiny)
        self.assertEqual(tiny | huge, tiny)

class VectorFromCasesTests(TestCase):
    """Tests of the VectorFromCases factory fuction."""

    def test_init(self):
        """VectorFromCases should return vector with 1 where string is ucase"""
        valid_strings = ['', 'a', 'X', 'aBc', 'Acb', 'abC', 'AAA', 'aaa', '@']
        results =       ['', '0', '1', '010', '100', '001', '111', '000', '0']

        for data, result in zip(valid_strings, results):
            self.assertEqual(VectorFromCases(data), LongBitvector(result))
            self.assertTrue(isinstance(VectorFromCases(data), LongBitvector))
            self.assertEqual(VectorFromCases(data, ShortBitvector), \
                ShortBitvector(result))
            self.assertTrue(isinstance(VectorFromCases(data, ShortBitvector), \
                                       ShortBitvector))

        v = VectorFromCases('aBC')
        self.assertTrue(isinstance(v, ImmutableBitvector))

        w = VectorFromCases('aBC', MutableBitvector)
        self.assertTrue(isinstance(w, MutableBitvector))
        self.assertEqual(w, Bitvector('011'))
        w[0] = 1
        self.assertEqual(w, Bitvector('111'))

class VectorFromMatchesTests(TestCase):
    """Tests of the VectorFromMatches factory function.
    
    Need to check all combinations of the following cases:
        1. Pattern is:
            -empty
            -single-character string
            -single-character regex
            -multi-character string
            -multi-character regex
            -regex that includes alternation

        2. String is:
            -empty
            -match at every position
            -match at some positions
            -match at no position

        3. Match is:
            -overlapping
            -non-overlapping
    """
    def testBothEmpty(self):
        """VectorFromMatches empty string/pattern should return empty vector"""
        self.assertEqual(str(VectorFromMatches('', '')), '')
        
    def testEmptyPattern(self):
        """VectorFromMatches empty pattern should return zeroes for len(string)"""
        sequences = ['', 'a', 'aa', 'aaaaaaaaaa']
        for s in sequences:
            vec = VectorFromMatches(s, '')
            self.assertEqual(str(vec), '0' * len(s))

    def testSingleBasePattern(self):
        """VectorFromMatches should match every matching char in string"""
        sequences = ['', 'a', 'b', 'aaa', 'bbb', 'aba', 'bab']
        a_matches = ['', '1', '0', '111', '000', '101', '010']
        b_matches = ['', '0', '1', '000', '111', '010', '101']
        
        for i, s in enumerate(sequences):
            vec = VectorFromMatches(s, 'a')
            self.assertEqual(str(vec), a_matches[i])
            vec = VectorFromMatches(s, 'b')
            self.assertEqual(str(vec), b_matches[i])

    def testMultiBasePattern(self):
        """VectorFromMatches should match multi-char string matches"""
        pattern = 'aba'
        sequences = ['','a', 'aba', 'abab', 'ababa', 'ababab', 'abaaba', 'aaba']
        overlap =   ['','0', '111', '1110', '11111', '111110', '111111', '0111']
        discrete =  ['','0', '111', '1110', '11100', '111000', '111111', '0111']
        for i, s in enumerate(sequences):
            vec = VectorFromMatches(s, pattern)
            self.assertEqual(str(vec), overlap[i])
            vec = VectorFromMatches(s, pattern, 1)
            self.assertEqual(str(vec), overlap[i])
            vec = VectorFromMatches(s, pattern, 0)
            self.assertEqual(str(vec), discrete[i])

    def testSingleBaseRegex(self):
        """VectorFromMatches should match every matching character in regex"""
        sequences  = ['', 'a', 'b', 'aaa', 'bbb', 'aba', 'bab', 'axb', 'xxx']
        a_matches  = ['', '1', '0', '111', '000', '101', '010', '100', '000']
        b_matches  = ['', '0', '1', '000', '111', '010', '101', '001', '000']
        ab_matches = ['', '1', '1', '111', '111', '111', '111', '101', '000']
        
        a = re.compile('a')
        b = re.compile('b')
        ab = re.compile('a|b')
        
        for i, s in enumerate(sequences):
            #test that a works as regex or list
            vec = VectorFromMatches(s, a)
            self.assertEqual(str(vec), a_matches[i])
            vec = VectorFromMatches(s, ['a'])
            self.assertEqual(str(vec), a_matches[i])
            
            #test that b works as regex or list
            vec = VectorFromMatches(s, b)
            self.assertEqual(str(vec), b_matches[i])
            vec = VectorFromMatches(s, ['b'])
            self.assertEqual(str(vec), b_matches[i])

            #test that [a or b] works as regex or list
            vec = VectorFromMatches(s, ab)
            self.assertEqual(str(vec), ab_matches[i])
            vec = VectorFromMatches(s, ['a', 'b'])
            self.assertEqual(str(vec), ab_matches[i])

    def testMultiBaseRegex(self):
        """VectorFromMatches should match every matching combination of chars"""
        sequences = ['aaabbb', 'aaaxbbb', 'ababab', 'abaabaabab']
        patterns =  ['aaa', 'bbb', 'aba', 'aaa|bbb', 'aaa|aab']

        overlap = { 'aaa'   :   [
                                    '111000',    #aaabbb
                                    '1110000',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'bbb'   :   [
                                    '000111',    #aaabbb
                                    '0000111',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'aba'   :   [
                                    '000000',    #aaabbb
                                    '0000000',   #aaaxbbb
                                    '111110',    #ababab
                                    '1111111110',#abaabaabab
                                ],
                    'aaa|bbb'   :   [
                                    '111111',    #aaabbb
                                    '1110111',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'aaa|aab'   :   [
                                    '111100',    #aaabbb
                                    '1110000',   #aaaxbbb
                                    '000000',    #ababab
                                    '0011111100',#abaabaabab
                                ]
                    }

        no_overlap = { 'aaa'   :   [
                                    '111000',    #aaabbb
                                    '1110000',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'bbb'   :   [
                                    '000111',    #aaabbb
                                    '0000111',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'aba'   :   [
                                    '000000',    #aaabbb
                                    '0000000',   #aaaxbbb
                                    '111000',    #ababab
                                    '1111111110',#abaabaabab
                                ],
                    'aaa|bbb'   :   [
                                    '111111',    #aaabbb
                                    '1110111',   #aaaxbbb
                                    '000000',    #ababab
                                    '0000000000',#abaabaabab
                                ],
                    'aaa|aab'   :   [
                                    '111000',    #aaabbb
                                    '1110000',   #aaaxbbb
                                    '000000',    #ababab
                                    '0011111100',#abaabaabab
                                ],
                    }
        for i, s in enumerate(sequences):
            for pat in patterns:
                regex = re.compile(pat)
                vec = VectorFromMatches(s, regex, 1)  #overlapping
                self.assertEqual(str(vec), overlap[pat][i])
                vec = VectorFromMatches(s, regex, 0)  #non-overlapping
                self.assertEqual(str(vec), no_overlap[pat][i])
    
    def test_type(self):
        """VectorFromMatches should return correct type of vector"""
        v = VectorFromMatches('a', 'a')
        self.assertEqual(v, Bitvector('1'))
        self.assertTrue(isinstance(v, ImmutableBitvector))
        v = VectorFromMatches('a', 'a', constructor=MutableBitvector)
        self.assertEqual(v, Bitvector('1'))
        v[0] = 0
        self.assertEqual(v, Bitvector('0'))
        self.assertTrue(isinstance(v, MutableBitvector))

class VectorFromRunsTests(TestCase):
    """Tests of the VectorFromRuns factory fuction."""

    def test_init(self):
        """VectorFromRuns should return vector with 1 where string is ucase"""
        empty = []
        empty_run = [[5,0]]
        one_run = [[5,3]]
        two_runs = [[5,1], [11,4]]
        overlap = [[5,3], [6,4]]

        tests = [empty, empty_run, one_run, two_runs, overlap]
        exp_15 = ['0'*15, '0'*15, '0'*5+'1'*3+7*'0', '0'*5+'1'+'0'*5+'1'*4, 
            '0'*5+'1'*5+'0'*5]
        for test, exp in zip(tests, exp_15):
            self.assertEqual(VectorFromRuns(test, 15), Bitvector(exp))
            self.assertEqual(VectorFromRuns(test, 0), Bitvector(''))
        #should fail if vec too short
        self.assertRaises(IndexError, VectorFromRuns, two_runs, 5)
        
class VectorFromSpansTests(TestCase):
    """Tests of the VectorFromSpans factory fuction."""

    def test_init(self):
        """VectorFromSpans hould return vector with 1 in correct spans"""
        empty = []
        empty_run = [[5,5]]
        one_run = [[5,8]]
        two_runs = [[5,6], [11,15]]
        overlap = [[5,8], [6,10]]

        tests = [empty, empty_run, one_run, two_runs, overlap]
        exp_15 = ['0'*15, '0'*15, '0'*5+'1'*3+7*'0', '0'*5+'1'+'0'*5+'1'*4,
            '0'*5+'1'*5+'0'*5]
        for test, exp in zip(tests, exp_15):
            self.assertEqual(VectorFromSpans(test, 15), Bitvector(exp))
            self.assertEqual(VectorFromSpans(test, 0), Bitvector(''))
        #should fail if vec too short
        self.assertRaises(IndexError, VectorFromSpans, two_runs, 5)

class VectorFromPositionsTests(TestCase):
    """Tests of the VectorFromPositions factory function."""
    def test_init(self):
        """VectorFromPositions should return correct vector"""
        empty = []
        first = [0]
        fifth = [4]
        several = [1,4,6,9]

        tests = [empty, first, fifth, several]
        exp_0 = ['']*4
        for test, exp in zip(tests, exp_0):
            self.assertEqual(VectorFromPositions(test, 0), Bitvector(exp))

        exp_5 = ['00000', '10000', '00001']
        for test, exp in zip(tests, exp_5):
            self.assertEqual(VectorFromPositions(test, 5), Bitvector(exp))

        exp_10 = ['0'*10, '1'+'0'*9, '00001'+'0'*5, '0100101001']
        for test, exp in zip(tests, exp_10):
            self.assertEqual(VectorFromPositions(test, 10), Bitvector(exp))
        self.assertRaises(IndexError, VectorFromPositions, [10], 5)

class PackedBasesTests(TestCase):
    """Tests of the PackedBases class."""
    
    def test_init(self):
        """PackedBases init should allow sequence recovery"""
        p = PackedBases()
        self.assertEqual(str(p), '')
        self.assertEqual(len(p), 0)
        p = PackedBases('uCaGGCAU')
        self.assertEqual(len(p), 16)
        self.assertEqual(str(p), 'UCAGGCAU')
        p = PackedBases('aaaAaaAaa')
        self.assertEqual(str(p), 'AAAAAAAAA')
        self.assertEqual(len(p), 18)
        for base in 'ucagUCAGtT':
            p = PackedBases(base)
            self.assertEqual(len(p), 2)
            self.assertEqual(str(p), base.upper().replace('T','U'))

    def test_str(self):
        """PackedBases str should respect RNA/DNA"""
        p = PackedBases('UCAGu')
        self.assertEqual(str(p), 'UCAGU')
        p.Rna = False
        self.assertEqual(str(p), 'TCAGT')
        p.Rna = True
        self.assertEqual(str(p), 'UCAGU')
        q = PackedBases('ucagu', Rna=False)
        self.assertEqual(str(q), 'TCAGT')
        p = PackedBases('T'*100, Rna=False)
        self.assertEqual(str(p), 'T'*100)
        p.Rna = True
        self.assertEqual(str(p), 'U'*100)

    def test_bit_ops(self):
        """PackedBases xor and bitcount should give correct # differences"""
        p = PackedBases('UCAGU')
        q = PackedBases('CAGGA')
        self.assertEqual((p^q).bitcount(), 5)
        r = PackedBases('CAAG')
        self.assertEqual((r^q).bitcount(), 1)
        
    
#main loop of program
if __name__ == "__main__":
    main()



"""The most commonly used constructors are available from this toplevel module.
The rest are in the subpackages: phylo, evolve, maths, draw, parse and format."""

import sys, os, re, cPickle
import numpy

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "Rob Knight", "Peter Maxwell",
                    "Jeremy Widmann", "Catherine Lozupone", "Matthew Wakefield",
                    "Edward Lang", "Greg Caporaso", "Mike Robeson",
                    "Micah Hamady", "Sandra Smit", "Zongzhi Liu",
                    "Andrew Butterfield", "Amanda Birmingham", "Brett Easton",
                    "Hua Ying", "Jason Carnes", "Raymond Sammut",
                    "Helen Lindsay", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

#SUPPORT2425
if sys.version_info < (2, 6):
    py_version = ".".join([str(n) for n in sys.version_info])
    raise RuntimeError("Python-2.6 or greater is required, Python-%s used." % py_version)

numpy_version = re.split("[^\d]", numpy.__version__)
numpy_version_info = tuple([int(i) for i in numpy_version if i.isdigit()])
if numpy_version_info < (1, 3):
    raise RuntimeError("Numpy-1.3 is required, %s found." % numpy_version)

version = __version__
version_info = tuple([int(v) for v in version.split(".") if v.isdigit()])


from cogent.util.table import Table as _Table
from cogent.parse.table import load_delimited, autogen_reader
from cogent.core.tree import TreeBuilder, TreeError
from cogent.parse.tree_xml import parse_string as tree_xml_parse_string
from cogent.parse.newick import parse_string as newick_parse_string
from cogent.core.alignment import SequenceCollection
from cogent.core.alignment import Alignment
from cogent.parse.sequence import FromFilenameParser
from cogent.parse.structure import FromFilenameStructureParser
#note that moltype has to be imported last, because it sets the moltype in
#the objects created by the other modules.
from cogent.core.moltype import ASCII, DNA, RNA, PROTEIN, STANDARD_CODON, \
        CodonAlphabet

def Sequence(moltype=None, seq=None, name=None, filename=None, format=None):
    if seq is None:
        for (a_name, a_seq) in FromFilenameParser(filename, format):
            if seq is None:
                seq = a_seq
                if name is None:
                    name = a_name
            else:
                raise ValueError("Multiple sequences in '%s'" % filename)
    if moltype is not None:
        seq = moltype.makeSequence(seq)
    elif not hasattr(seq, 'MolType'):
        seq = ASCII.makeSequence(seq)
    if name is not None:
        seq.Name = name
    return seq

def LoadSeqs(filename=None, format=None, data=None, moltype=None,
            name=None, aligned=True, label_to_name=None, parser_kw={},
            constructor_kw={}, **kw):
    """Initialize an alignment or collection of sequences.
    
    Arguments:
    - filename: name of the sequence file
    - format: format of the sequence file
    - data: optional explicit provision of sequences
    - moltype: the MolType, eg DNA, PROTEIN
    - aligned: set True if sequences are already aligned and have the same
      length, results in an Alignment object. If False, a SequenceCollection
      instance is returned instead. If callable, will use as a constructor
      (e.g. can pass in DenseAlignment or CodonAlignment).
    - label_to_name: function for converting original name into another
      name. Default behavior is to preserve the original FASTA label and
      comment. 
      To remove all FASTA label comments, and pass in only the label, pass in: 
            label_to_name=lambda x: x.split()[0]
      To look up names in a dict, pass in:
            label_to_name = lambda x: d.get(x, default_name)
      ...where d is a dict that's in scope, and default_name is what you want
      to assign any sequence that isn't in the dict.
    
    If format is None, will attempt to infer format from the filename
    suffix. If label_to_name is None, will attempt to infer correct
    conversion from the format.
    """
    
    if filename is None:
        assert data is not None
        assert format is None
        assert not kw, kw
    else:
        assert data is None, (filename, data)
        data = list(FromFilenameParser(filename, format, **parser_kw))

    # the following is a temp hack until we have the load API sorted out.
    if aligned: #if callable, call it -- expect either f(data) or bool
        if hasattr(aligned, '__call__'):
            return aligned(data=data, MolType=moltype, Name=name,
                label_to_name=label_to_name, **constructor_kw)
        else:   #was not callable, but wasn't False
            return Alignment(data=data, MolType=moltype, Name=name,
                label_to_name=label_to_name, **constructor_kw)
    else:   #generic case: return SequenceCollection
        return SequenceCollection(data, MolType=moltype, Name=name,
            label_to_name=label_to_name, **constructor_kw)

def LoadStructure(filename, format=None, parser_kw={}):
    """Initialize a Structure from data contained in filename.
    Arguments:
        - filename: name of the filename to create structure from.
        - format: the optional file format extension.
        - parser_kw: optional keyword arguments for the parser."""
    # currently there is no support for string-input
    assert filename is not None, 'No filename given.'
    return FromFilenameStructureParser(filename, format, **parser_kw)


def LoadTable(filename=None, sep=',', reader=None, header=None, rows=None,
            row_order=None, digits=4, space=4, title='', missing_data='',
            max_width = 1e100, row_ids=False, legend='', column_templates=None,
            dtype=None, static_column_types=False, limit=None, **kwargs):
    """
    Arguments:
    - filename: path to file containing a pickled table
    - sep: the delimiting character between columns
    - reader: a parser for reading filename. This approach assumes the first
      row returned by the reader will be the header row.
    - static_column_types: if True, and reader is None, identifies columns
      with a numeric data type (int, float) from the first non-header row.
      This assumes all subsequent entries in that column are of the same type.
      Default is False.
    - header: column headings
    - rows: a 2D dict, list or tuple. If a dict, it must have column
      headings as top level keys, and common row labels as keys in each
      column.
    - row_order: the order in which rows will be pulled from the twoDdict
    - digits: floating point resolution
    - space: number of spaces between columns or a string
    - title: as implied
    - missing_data: character assigned if a row has no entry for a column
    - max_width: maximum column width for printing
    - row_ids: if True, the 0'th column is used as row identifiers and keys
      for slicing.
    - legend: table legend
    - column_templates: dict of column headings: string format templates
      or a function that will handle the formatting.
    - dtype: optional numpy array typecode.
    - limit: exits after this many lines. Only applied for non pickled data
      file types.
    """
    # 
    if filename is not None and not (reader or static_column_types):
        if filename[filename.rfind(".")+1:] == 'pickle':
            f = file(filename, 'U')
            loaded_table = cPickle.load(f)
            f.close()
            return _Table(**loaded_table)

        sep = sep or kwargs.pop('delimiter', None)
        header, rows, loaded_title, legend = load_delimited(filename,
                                    delimiter = sep, limit=limit, **kwargs)
        title = title or loaded_title
    elif filename and (reader or static_column_types):
        f = file(filename, "r")
        if not reader:
            reader = autogen_reader(f, sep, limit=limit,
                        with_title=kwargs.get('with_title', False))
        
        rows = [row for row in reader(f)]
        f.close()
        header = rows.pop(0)

    table = _Table(header=header, rows=rows, digits=digits, row_order=row_order,
                title=title,
                dtype=dtype, column_templates=column_templates, space=space,
                missing_data=missing_data, max_width=max_width, row_ids=row_ids,
                legend=legend)

    return table

def LoadTree(filename=None, treestring=None, tip_names=None, format=None, \
    underscore_unmunge=False):

    """Constructor for tree.
    
    Arguments, use only one of:
        - filename: a file containing a newick or xml formatted tree.
        - treestring: a newick or xml formatted tree string.
        - tip_names: a list of tip names.

    Note: underscore_unmunging is turned off by default, although it is part
    of the Newick format. Set underscore_unmunge to True to replace underscores
    with spaces in all names read.
    """

    if filename:
        assert not (treestring or tip_names)
        treestring = open(filename).read()
        if format is None and filename.endswith('.xml'):
            format = "xml"
    if treestring:
        assert not tip_names
        if format is None and treestring.startswith('<'):
            format = "xml"
        if format == "xml":
            parser = tree_xml_parse_string
        else:
            parser = newick_parse_string
        tree_builder = TreeBuilder().createEdge
        #FIXME: More general strategy for underscore_unmunge
        if parser is newick_parse_string:
            tree = parser(treestring, tree_builder, \
                    underscore_unmunge=underscore_unmunge)
        else:
            tree = parser(treestring, tree_builder)
        if not tree.NameLoaded:
            tree.Name = 'root'
    elif tip_names:
        tree_builder = TreeBuilder().createEdge
        tips = [tree_builder([], tip_name, {}) for tip_name in tip_names]
        tree = tree_builder(tips, 'root', {})
    else:
        raise TreeError, 'filename or treestring not specified'
    return tree




#!/usr/bin/env python

from cogent.app.util import CommandLineApplication,\
    CommandLineAppResult, ResultPath
from cogent.app.parameters import Parameter, FlagParameter, ValuedParameter,\
    MixedParameter,Parameters, _find_synonym, is_not_None

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"

class comRNA(CommandLineApplication):
    """Application controller comRNA v1.80

    for comRNA options type comRNA at the promt
    """

    _parameters = {
        '-L':ValuedParameter(Prefix='',Name='L',Value=None,Delimiter=' '),
        '-E':ValuedParameter(Prefix='',Name='E',Value=None,Delimiter=' '),
        '-S':ValuedParameter(Prefix='',Name='S',Value=None,Delimiter=' '),
        '-Sh':ValuedParameter(Prefix='',Name='Sh',Value=None,Delimiter=' '),
        '-Sl':ValuedParameter(Prefix='',Name='Sl',Value=None,Delimiter=' '),
        '-P':ValuedParameter(Prefix='',Name='P',Value=None,Delimiter=' '),
        '-n':ValuedParameter(Prefix='',Name='n',Value=None,Delimiter=' '),
        '-x':ValuedParameter(Prefix='',Name='x',Value=None,Delimiter=' '),
        '-m':ValuedParameter(Prefix='',Name='m',Value=None,Delimiter=' '),
        '-tp':ValuedParameter(Prefix='',Name='tp',Value=None,Delimiter=' '),
        '-ts':ValuedParameter(Prefix='',Name='ts',Value=None,Delimiter=' '),
        '-a':ValuedParameter(Prefix='',Name='a',Value=None,Delimiter=' '),
        '-o':ValuedParameter(Prefix='',Name='o',Value=None,Delimiter=' '),
        '-c':ValuedParameter(Prefix='',Name='c',Value=None,Delimiter=' '),
        '-j':ValuedParameter(Prefix='',Name='j',Value=None,Delimiter=' '),
        '-r':ValuedParameter(Prefix='',Name='r',Value=None,Delimiter=' '),
        '-f':ValuedParameter(Prefix='',Name='f',Value=None,Delimiter=' '),
        '-v':ValuedParameter(Prefix='',Name='v',Value=None,Delimiter=' '),
        '-g':ValuedParameter(Prefix='',Name='g',Value=None,Delimiter=' '),
        '-d':ValuedParameter(Prefix='',Name='d',Value=None,Delimiter=' '),
        '-wa':ValuedParameter(Prefix='',Name='wa',Value=None,Delimiter=' '),
        '-wb':ValuedParameter(Prefix='',Name='wb',Value=None,Delimiter=' '),
        '-wc':ValuedParameter(Prefix='',Name='wc',Value=None,Delimiter=' '),
        '-wd':ValuedParameter(Prefix='',Name='wd',Value=None,Delimiter=' '),
        '-we':ValuedParameter(Prefix='',Name='we',Value=None,Delimiter=' '),
        '-pk':ValuedParameter(Prefix='',Name='pk',Value=None,Delimiter=' '),
        '-pg':ValuedParameter(Prefix='',Name='pg',Value=None,Delimiter=' '),
        '-pd':ValuedParameter(Prefix='',Name='pd',Value=None,Delimiter=' '),
        '-ps':ValuedParameter(Prefix='',Name='ps',Value=None,Delimiter=' '),
        '-pc':ValuedParameter(Prefix='',Name='pc',Value=None,Delimiter=' ')}

    _command = 'comRNA'
    _input_handler = '_input_as_string'





#!/usr/bin/env python
# test_mothur.py


from __future__ import with_statement
from cStringIO import StringIO
from os import remove, rmdir
from tempfile import mkdtemp, mkstemp, NamedTemporaryFile
from cogent.util.unit_test import TestCase, main
from cogent.app.mothur import (
    Mothur, mothur_from_file,
    MothurClassifySeqs, mothur_classify_file
    )


__author__ = "Kyle Bittinger"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Kyle Bittinger", "Jose Carlos Clemente Litran"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Kyle Bittinger"
__email__ = "kylebittinger@gmail.com"
__status__ = "Development"


class MothurTests(TestCase):
    def setUp(self):
        self.small_fasta = (
            '>aaaaaa\nTAGGCTCTGATATAATAGCTCTC---------\n'
            '>cccccc\n------------TGACTACGCAT---------\n'
            '>bbbbbb\n----TATCGCTTCGACGATTCTCTGATAGAGA\n'
            )
        self.small_otus = (
            'unique\t3\taaaaaa\tcccccc\tbbbbbb\t\n'
            '0.62\t2\taaaaaa\tbbbbbb,cccccc\t\n'
            '0.67\t1\tbbbbbb,cccccc,aaaaaa\t\n'
            )
        self.small_otus_parsed = [
            (float('0'), [['aaaaaa'], ['cccccc'], ['bbbbbb']]),
            (float('0.62'), [['aaaaaa'], ['bbbbbb', 'cccccc']]),
            (float('0.67'), [['bbbbbb', 'cccccc', 'aaaaaa']]),
            ]
        self.complement_fasta = (
            '>a\n--AGGGGTAATAA--\n'
            '>b\n--TTATTACCCCT--\n'
            '>c\n-------AAAAAA--\n'
            )
        self.complement_otus = (
            'unique\t3\ta\tb\tc\t\n'
            '0.43\t2\tc,a\tb\t\n'
            '1.00\t1\tb,c,a\t\n'
            )

    def test_get_help(self):
        """Mothur.getHelp() should return help string"""
        expected_help = (
            'See manual, available on the MOTHUR wiki:\n'
            'http://schloss.micro.umass.edu/mothur/'
            )
        self.assertEqual(Mothur.getHelp(), expected_help)

    def test_compile_mothur_script(self):
        """Mothur._compile_mothur_script() should return valid Mothur script"""
        app = Mothur()
        app._input_filename = 'test.fasta'
        observed_script = app._compile_mothur_script()
        expected_script = (
            '#unique.seqs(fasta=test.fasta); '
            'dist.seqs(fasta=test.unique.fasta); '
            'read.dist(column=test.unique.dist, name=test.names); '
            'cluster(method=furthest)')
        self.assertEqual(observed_script, expected_script)

    def test_get_result_paths(self):
        """Mothur._get_result_paths() should guess correct output paths"""
        app = Mothur()
        app._input_filename = 'test.fasta'
        observed_paths = {
            'distance matrix': app._derive_dist_path(),
            'otu list': app._derive_list_path(),
            'rank abundance': app._derive_rank_abundance_path(),
            'species abundance': app._derive_species_abundance_path(),
            'unique names': app._derive_names_path(),
            'unique seqs': app._derive_unique_path(),
            }
        expected_paths = {
            'distance matrix': 'test.unique.dist',
            'otu list': 'test.unique.fn.list',
            'rank abundance': 'test.unique.fn.rabund',
            'species abundance': 'test.unique.fn.sabund',
            'unique names': 'test.names',
            'unique seqs': 'test.unique.fasta',
            }
        self.assertEqual(observed_paths, expected_paths)

    def test_working_directory(self):
        """Mothur.WorkingDir attribute should not be cast to FilePath object"""
        app = Mothur(WorkingDir='/tmp')
        self.assertEquals(str(app.WorkingDir), '/tmp')

    def test_call_with_multiline_string(self):
        """Mothur.__call__() should return correct otu's for input as single string"""
        app = Mothur()
        result = app(self.small_fasta)
        observed_otus = result['otu list'].read()
        self.assertEquals(observed_otus, self.small_otus)
        result.cleanUp()

    def test_call_with_lines(self):
        """Mothur.__call__() should return correct otu's for input as lines"""
        lines = self.small_fasta.split('\n')
        app = Mothur(InputHandler='_input_as_lines')
        result = app(lines)
        observed_otus = result['otu list'].read()
        self.assertEquals(observed_otus, self.small_otus)
        result.cleanUp()

    def test_call_with_path(self):
        """Mothur.__call__() should return correct otu's for input as path"""
        working_dir = mkdtemp()
        _, filename = mkstemp(dir=working_dir, suffix='.fasta')
        with open(filename, 'w') as f:
            f.write(self.small_fasta)
        app = Mothur(InputHandler='_input_as_path', WorkingDir=working_dir)
        result = app(filename)
        observed_otus = result['otu list'].read()
        self.assertEquals(observed_otus, self.small_otus)
        remove(filename)
        result.cleanUp()
        rmdir(working_dir)

    def test_call_with_working_dir(self):
        """Mothur.__call__() should return correct otu's when input dir is changed"""
        working_dir = mkdtemp()
        app = Mothur(WorkingDir=working_dir)
        result = app(self.small_fasta)
        observed_otus = result['otu list'].read()
        self.assertEquals(observed_otus, self.small_otus)
        result.cleanUp()
        rmdir(working_dir)

    def test_call_with_complement(self):
        """Mothur.__call__() should return correct otu's for input sequences which are reverse complements"""
        app = Mothur()
        result = app(self.complement_fasta)
        observed_otus = result['otu list'].read()
        self.assertEquals(observed_otus, self.complement_otus)
        result.cleanUp()

    def test_mothur_from_file(self):
        """mothur_from_file() should return parsed otus"""
        f = StringIO(self.small_fasta)
        f.seek(0)
        parsed_otus = mothur_from_file(f)
        self.assertEquals(parsed_otus, self.small_otus_parsed)


class TestMothurClassifySeqs(TestCase):
    def setUp(self):
        self.ref_file = NamedTemporaryFile()
        self.ref_file.write(mothur_ref_seqs)
        self.ref_file.seek(0)

        self.tax_file = NamedTemporaryFile()
        self.tax_file.write(mothur_taxonomy)
        self.tax_file.seek(0)

    def test_app(self):
        app = MothurClassifySeqs({
            'reference': self.ref_file.name,
            'taxonomy': self.tax_file.name,
            })
        res = app(mothur_seqs)
        assignments = res['assignments'].read()
        self.assertEqual(assignments, mothur_assignments)

        summary = res['summary'].read()
        self.assertEqual(summary, mothur_summary)
        

    def test_format_function_arguments(self):
        app = MothurClassifySeqs({
            'reference': '/home/myuser/ref-seqs.fasta',
            'taxonomy': '/home/MyUser/data/tax.txt',
            'cutoff': 80,
            })
        obs_args = app._format_function_arguments(
            ['reference', 'taxonomy', 'cutoff', 'iters'])
        exp_args = (
            "reference=/home/myuser/ref\\-seqs.fasta, "
            "taxonomy=/home/MyUser/data/tax.txt, cutoff=80")
        self.assertEqual(obs_args, exp_args)

    def test_compile_mothur_script(self):
        app = MothurClassifySeqs({
            'reference': '/home/myuser/ref-seqs.fasta',
            'taxonomy': '/home/MyUser/data/tax.txt',
            'cutoff': 80,
            })
        app._input_filename = "/my/input.fasta"
        exp_script = (
            "#classify.seqs(fasta=/my/input.fasta, "
            "reference=/home/myuser/ref\-seqs.fasta, "
            "taxonomy=/home/MyUser/data/tax.txt, "
            "cutoff=80)")
        self.assertEqual(app._compile_mothur_script(), exp_script)

    def test_mothur_classify_file(self):
        query_file = StringIO(mothur_seqs)
        res = mothur_classify_file(
            query_file,  self.ref_file.name, self.tax_file.name)
        exp_res = {
            'A': (['k__Bacteria', 'p__Firmicutes', 'c__Clostridia',
                   'o__Clostridale', 'f__Eubacteriaceae', 'g__Eubacterium',
                   's__Eubacteriumfoedans'], 1.0),
            'Very': (['k__Bacteria', 'p__Bacteriodetes'], 1.0),
            '01': (['k__Bacteria', 'p__Firmicutes'], 1.0),
            }
        self.assertEqual(res, exp_res)

    def test_unclassifiable_sequence(self):
        query_file = StringIO(
            ">MostlyTs\nTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT"
            "TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTATTTTTTTTTTTTTTTTTTTTTTTTTTTTTT"
            "TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT\n")
        res = mothur_classify_file(
            query_file,  self.ref_file.name, self.tax_file.name)
        exp_res = {
            'MostlyTs': (['Unknown'], 0.0),
            }
        self.assertEqual(res, exp_res)


mothur_assignments = """\
01	k__Bacteria(100);p__Firmicutes(100);unclassified;unclassified;unclassified;unclassified;unclassified;
A	k__Bacteria(100);p__Firmicutes(100);c__Clostridia(100);o__Clostridale(100);f__Eubacteriaceae(100);g__Eubacterium(100);s__Eubacteriumfoedans(100);
Very	k__Bacteria(100);p__Bacteriodetes(100);unclassified;unclassified;unclassified;unclassified;unclassified;
"""

mothur_summary = """\
taxlevel	 rankID	 taxon	 daughterlevels	 total	
0	0	Root	1	3	
1	0.1	k__Bacteria	2	3	
2	0.1.1	p__Bacteriodetes	1	1	
3	0.1.1.1	unclassified	1	1	
4	0.1.1.1.1	unclassified	1	1	
5	0.1.1.1.1.1	unclassified	1	1	
6	0.1.1.1.1.1.1	unclassified	1	1	
7	0.1.1.1.1.1.1.1	unclassified	0	1	
2	0.1.2	p__Firmicutes	2	2	
3	0.1.2.1	c__Clostridia	1	1	
4	0.1.2.1.1	o__Clostridale	1	1	
5	0.1.2.1.1.1	f__Eubacteriaceae	1	1	
6	0.1.2.1.1.1.1	g__Eubacterium	1	1	
7	0.1.2.1.1.1.1.1	s__Eubacteriumfoedans	0	1	
3	0.1.2.2	unclassified	1	1	
4	0.1.2.2.1	unclassified	1	1	
5	0.1.2.2.1.1	unclassified	1	1	
6	0.1.2.2.1.1.1	unclassified	1	1	
7	0.1.2.2.1.1.1.1	unclassified	0	1	
"""

mothur_seqs = """\
>01
GGAGTCTGGGCCGGTGTCGTCAAGGTCCCAATCTGGCTGGTCGGTCTCTCAACCCAGCTACCCATCATTGCCTTGGTAGGCCGTTACCCACCAACAAGCTAACAGGCCGCGGGCCCATCCCTCTCCGCCGGAGCTTTCTCGAGTCTTCCATGCGGAAGTCCCGAAGTATTCGGTATTATCCACGGTTTCCCGTGGCTATCCCAATGAGAGGGGCAGGTTGCCCACGTGTTACTCAGCCGTTCGCCACTTTATACACACCCGAAGGTGCTTTAATCGTTCGACTTGCATGTGTTAGGCGCGCCGCCAGCGTTCATC
>A
GGAGTCTGGGCCGTGTCTCAGTCCCAATGTGGCCGATCACCCTCTCAGGTCGGCTATGCATCACGGCCTTGGTGAGCCGTTACCTCACCAACTAGCTAATGCACCGCGGGTCCATCCATCAGCAGAAGCTTGCGCCTCTTTTCCTCTTCAAACCATGCGGTTCGAAGACCTATGCGGTTTTAGCATCCGTTTCCGAATGTTATCCCCCTCTGATGGGCAGGTTACCCACGTGTTACTCACCCGTTCGCCACTAGATTGACCAGTGCAAGCACCGGTCGCTCTCGTTCGACTTGCATGTATTAGGCACGCCGCCAGCGTTCGTC
>Very long seq name with many spaces!
GGAGTCTGGACCGTGTCTCAGTTCCAGTGTGACTGATCATCCTCTCAGACCAGTTATGCGTCATAGCCTTGGTGAGCCATTACCTCACCAACTAGCTGATACAATATAGCCTCATCCTACACCGAAAAACTTTCCCTATCTAACTTATGTTAGAGAGGAGTATAGAGTATTAGCAGTCGTTTCCAACTGTTGTCCTCTAGTGTAGGGCAGATTAGCTACACATTACTCACCCGTGCGCCACTAACTCATAAGAGCAAGCTCTTACTTGTCCGTTCGACTTGCATGTATTAGGCACGCCGCCAGCGTTCACT
"""

mothur_ref_seqs = """\
>ref1
GGAGTCTGGGCCGGTGTCGTCAAGGTCCCAATCTGGCTGGTCGGTCTCTCTGGTAGGCCGTTACCCACCAACAAGCTAACAGGCCGCGGGCCCATCCCTCTCCGCCGGAGCTTTCTCGAGTCTTCCATGCGGAAGTCCCTGCGGAAGTCCCGAAGTATTCGGTATTATCCACGGTTTCCCGTGGCTATCCCAATGAGAGGGGCAGGTTGCCCACGTGTTACTCAGCCGTTCGCCACTTTATACACACCCGAAGGTGCTTTAATCGTTCGACTTGCATGTGTTAGGCGCGCCGCCAGCGTTCATC
>ref2
GGAGTCTGGGCCGTGTCTCAGTCCCAATGTGGCCGATCACCCTCTCAGGTCGGCTATGCATCACGGCCTTGGTGAGCCGTTACCTCACCAACTAGCTACTCTTTTCCTCTTCAAACCATGCGGTTCGAAGACCTATGCGGTTTTAGCATCCGTAAACTTTCCCTATCTAACTTATGTTAGAGAGGAGTATAGAGTATTAGCAGTCGTTTCCAACTTCCGAATGTTATCCCCCTCTGATGGGCAGGTTACCCACGTGTTACTCACCCGTTCGCCACTAGATTGACCAGTGCAAGCACCGGTCGCTCTCGTTCGACTTGCATGTATTAGGCACGCCGCCAGCGTTCGTC
>3333
GGAGTCTGGACCGTGTCTCAGTTCCAGTGTGACTGATCATCCTCTCAGACAGTTATGCGTCATAGCCTTGGTGAGCCATTACCTCACCAACTAGCTGATACAATATAGCCTCATCCTACACCGAAAAACTTTCCCTATCTCTTATGTTAGAGAGGAGTATAGAGTATTAGCAGTCGTTTCCAACTGTTGTCCTCTAGTGTAGGGCAGATTAGCACACATTACTCACCCGTGCGCCACTAACTCATAAGAGCAAGCTCTTACTTGTCCGTTCGACTTGCATGTATTAGGCACGCCGCCAGCGTTCACT
"""

mothur_taxonomy = """\
ref1	k__Bacteria;p__Firmicutes;
ref2	k__Bacteria;p__Firmicutes;c__Clostridia;o__Clostridale;f__Eubacteriaceae;g__Eubacterium;s__Eubacteriumfoedans;
3333	k__Bacteria;p__Bacteriodetes;
"""

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Tests for the rdp_classifier_2.0.1 application controller"""

import os
from cStringIO import StringIO
from os import getcwd, environ, remove, listdir
from shutil import rmtree

import tempfile
from cogent.app.util import ApplicationNotFoundError, ApplicationError,\
    get_tmp_filename
from cogent.app.rdp_classifier import (
    RdpClassifier, RdpTrainer, assign_taxonomy, train_rdp_classifier,
    train_rdp_classifier_and_assign_taxonomy, parse_rdp_assignment
    )
from cogent.util.unit_test import TestCase, main

__author__ = "Kyle Bittinger"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Kyle Bittinger"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Kyle Bittinger"
__email__ = "kylebittinger@gmail.com"
__status__ = "Prototype"

class RdpClassifierTests(TestCase):
    def setUp(self):
        # fetch user's RDP_JAR_PATH
        if 'RDP_JAR_PATH' in environ:
            self.user_rdp_jar_path = environ['RDP_JAR_PATH']
        else:
            self.user_rdp_jar_path = 'rdp_classifier-2.2.jar'
        self.output_file = tempfile.NamedTemporaryFile()

    def test_default_java_vm_parameters(self):
        """RdpClassifier should store default arguments to Java VM."""
        a = RdpClassifier()
        self.assertContains(a.Parameters, '-Xmx')
        self.assertEqual(a.Parameters['-Xmx'].Value, '1000m')

    def test_parameters_list(self):
        a = RdpClassifier()
        parameters = a.Parameters.keys()
        parameters.sort()
        self.assertEqual(parameters, ['-Xmx', '-f', '-o', '-t'])

    def test_assign_jvm_parameters(self):
        """RdpCalssifier should pass alternate parameters to Java VM."""
        app = RdpClassifier()
        app.Parameters['-Xmx'].on('75M')
        exp = ''.join([
            'cd "', getcwd(), '/"; java -Xmx75M -jar "',
            self.user_rdp_jar_path, '" -q'])
        self.assertEqual(app.BaseCommand, exp)

    def test_basecommand_property(self):
        """RdpClassifier BaseCommand property should use overridden method."""
        app = RdpClassifier()
        self.assertEqual(app.BaseCommand, app._get_base_command())

    def test_base_command(self):
        """RdpClassifier should return expected shell command."""
        app = RdpClassifier()
        exp = ''.join([
            'cd "', getcwd(), '/"; java -Xmx1000m -jar "',
            self.user_rdp_jar_path, '" -q'])
        self.assertEqual(app.BaseCommand, exp)
        
    def test_change_working_dir(self):
        """RdpClassifier should run program in expected working directory."""
        test_dir = '/tmp/RdpTest'

        app = RdpClassifier(WorkingDir=test_dir)
        exp = ''.join([
            'cd "', test_dir, '/"; java -Xmx1000m -jar "',
            self.user_rdp_jar_path, '" -q'])
        self.assertEqual(app.BaseCommand, exp)

        rmtree(test_dir)

    def test_sample_fasta(self):
        """RdpClassifier should classify its own sample data correctly"""
        test_dir = '/tmp/RdpTest'
        app = RdpClassifier(WorkingDir=test_dir)
        _, output_fp = tempfile.mkstemp(dir=test_dir)
        app.Parameters['-o'].on(output_fp)        

        results = app(StringIO(rdp_sample_fasta))

        assignment_toks = results['Assignments'].readline().split('\t')

        self.assertEqual(assignment_toks[0], 'X67228')
        lineage = [x.strip('"') for x in assignment_toks[2::3]]
        self.assertEqual(lineage, [
            'Root', 'Bacteria', 'Proteobacteria', 'Alphaproteobacteria',
            'Rhizobiales', 'Rhizobiaceae', 'Rhizobium'])
        rmtree(test_dir)


class RdpTrainerTests(TestCase):
    """Tests of the trainer for the RdpClassifier app
    """

    def setUp(self):
        self.reference_file = StringIO(rdp_training_sequences)
        self.reference_file.seek(0)

        self.taxonomy_file = tempfile.NamedTemporaryFile(
            prefix="RdpTaxonomy", suffix=".txt")
        self.taxonomy_file.write(rdp_training_taxonomy)
        self.taxonomy_file.seek(0)

        self.training_dir = tempfile.mkdtemp(prefix='RdpTrainer_output_')

    def tearDown(self):
        rmtree(self.training_dir)

    def test_call(self):
        app = RdpTrainer()
        app.Parameters['taxonomy_file'].on(self.taxonomy_file.name)
        app.Parameters['model_output_dir'].on(self.training_dir)
        results = app(self.reference_file)

        exp_file_list = [
            'bergeyTrainingTree.xml', 'genus_wordConditionalProbList.txt',
            'logWordPrior.txt', 'RdpClassifier.properties',
            'wordConditionalProbIndexArr.txt',
            ]
        obs_file_list = listdir(self.training_dir)
        exp_file_list.sort()
        obs_file_list.sort()
        self.assertEqual(obs_file_list, exp_file_list)

        autogenerated_headers = {
            'bergeyTree': 'bergeyTrainingTree',
            'probabilityList': 'genus_wordConditionalProbList',
            'wordPrior': 'logWordPrior',
            'probabilityIndex': 'wordConditionalProbIndexArr',
            }
        for id, basename in autogenerated_headers.iteritems():
            obs_header = results[id].readline()
            exp_header = exp_training_header_template % basename
            self.assertEqual(exp_header, obs_header)


class RdpWrapperTests(TestCase):
    """ Tests of RDP classifier wrapper functions
    """
    def setUp(self):
        self.num_trials = 10
        
        self.test_input1 = rdp_test_fasta.split('\n')
        self.expected_assignments1 = rdp_expected_out
        
        # Files for training
        self.reference_file = StringIO(rdp_training_sequences)
        self.reference_file.seek(0)

        self.taxonomy_file = StringIO(rdp_training_taxonomy)
        self.taxonomy_file.seek(0)

        self.training_dir = tempfile.mkdtemp(prefix='RdpTrainer_output_')

        # Sequences for trained classifier
        self.test_trained_input = rdp_trained_fasta.split("\n")

    def tearDown(self):
        rmtree(self.training_dir)

    def test_parse_rdp_assignment(self):
        seqid, direction, assignments = parse_rdp_assignment(
            "X67228\t\t"
            "Root\tnorank\t1.0\t"
            "Bacteria\tdomain\t1.0\t"
            "\"Proteobacteria\"\tphylum\t1.0\t"
            "Alphaproteobacteria\tclass\t0.9\t"
            "Rhizobiales\torder\t0.9\t"
            "Rhizobiaceae\tfamily\t0.47\t"
            "Rhizobium\tgenus\t0.46")
        self.assertEqual(seqid, "X67228")

    def test_assign_taxonomy_short_sequence(self):
        """assign_taxonomy should return Unclassifiable if sequence is too short
        """
        assignments = assign_taxonomy([
            '>MySeq 1',
            'TTCCGGTTGATCCTGCCGGACCCGACTGCTATCCGGA',
            ])
        self.assertEqual(assignments, {'MySeq 1': ('Unassignable', 1.0)})
        
    def test_assign_taxonomy(self):
        """assign_taxonomy wrapper functions as expected 
        
        This test may fail periodicially, but failure should be rare.
        """
        unverified_seq_ids = set(self.expected_assignments1.keys())
        for i in range(self.num_trials):
            obs_assignments = assign_taxonomy(self.test_input1)
            for seq_id in list(unverified_seq_ids):
                obs_lineage, obs_confidence = obs_assignments[seq_id]
                exp_lineage = self.expected_assignments1[seq_id]
                if (obs_lineage == exp_lineage):
                    unverified_seq_ids.remove(seq_id)
            if not unverified_seq_ids:
                break

        messages = []
        for seq_id in unverified_seq_ids:
            messages.append("Unable to verify %s trials" % self.num_trials)
            messages.append("  Sequence ID: %s" % seq_id)
            messages.append("  Expected: %s" % self.expected_assignments1[seq_id])
            messages.append("  Observed: %s" % obs_assignments[seq_id][0])
            messages.append("  Confidence: %s" % obs_assignments[seq_id][1])
        
        # make sure all taxonomic results were correct at least once
        self.assertFalse(unverified_seq_ids, msg='\n'.join(messages))
            
    def test_assign_taxonomy_alt_confidence(self):
        """assign_taxonomy wrapper functions as expected with alt confidence
        """
        obs_assignments = assign_taxonomy(
            self.test_input1, min_confidence=0.95)            

        for seq_id, assignment in obs_assignments.items():
            obs_lineage, obs_confidence = assignment
            exp_lineage = self.expected_assignments1[seq_id]
            message = "Sequence ID: %s, assignment: %s" % (seq_id, assignment)
            self.assertTrue(
                exp_lineage.startswith(obs_lineage) or \
                (obs_lineage == "Unclassified"),
                msg=message,
                )
            self.assertTrue(obs_confidence >= 0.95, msg=message)
            
    def test_assign_taxonomy_file_output(self):
        """ assign_taxonomy wrapper writes correct file output when requested
        
            This function tests for sucessful completion of assign_taxonomy
             when writing to file, that the lines in the file roughly look
             correct by verifying how many are written (by zipping with 
             expected), and that each line starts with the correct seq id.
             Actual testing of taxonomy data is performed elsewhere.
        
        """
        output_fp = get_tmp_filename(\
         prefix='RDPAssignTaxonomyTests',suffix='.txt')
        # convert the expected dict to a list of lines to match 
        # file output
        expected_file_headers = self.expected_assignments1.keys()
        expected_file_headers.sort()
        
        actual_return_value = assign_taxonomy(\
         self.test_input1,min_confidence=0.95,output_fp=output_fp)
        
        actual_file_output = list(open(output_fp))
        actual_file_output.sort()

        # remove the output_fp before running the tests, so if they
        # fail the output file is still cleaned-up
        remove(output_fp)
        
        # None return value on write to file
        self.assertEqual(actual_return_value,None)
        
        # check that each line starts with the correct seq_id -- not 
        # checking the taxonomies or confidences here as these are variable and
        # tested elsewhere
        for a,e in zip(actual_file_output,expected_file_headers):
            self.assertTrue(a.startswith(e))

    def test_train_rdp_classifier(self):
        results = train_rdp_classifier(
            self.reference_file, self.taxonomy_file, self.training_dir)

        exp_file_list = [
            'bergeyTrainingTree.xml', 'genus_wordConditionalProbList.txt',
            'logWordPrior.txt', 'RdpClassifier.properties',
            'wordConditionalProbIndexArr.txt',
            ]
        obs_file_list = listdir(self.training_dir)
        exp_file_list.sort()
        obs_file_list.sort()
        self.assertEqual(obs_file_list, exp_file_list)

        autogenerated_headers = {
            'bergeyTree': 'bergeyTrainingTree',
            'probabilityList': 'genus_wordConditionalProbList',
            'wordPrior': 'logWordPrior',
            'probabilityIndex': 'wordConditionalProbIndexArr',
            }
        for id, basename in autogenerated_headers.iteritems():
            obs_header = results[id].readline()
            exp_header = exp_training_header_template % basename
            self.assertEqual(exp_header, obs_header)

    def test_train_rdp_classifier_and_assign_taxonomy(self):
        obs = train_rdp_classifier_and_assign_taxonomy(self.reference_file,
            self.taxonomy_file, self.test_trained_input, min_confidence=0.80,
            model_output_dir=self.training_dir)
        exp = {'X67228': (
            'Bacteria;Proteobacteria;Alphaproteobacteria;Rhizobiales;'
            'Rhizobiaceae;Rhizobium', 1.0
            )}
        self.assertEqual(obs, exp)

    def test_train_rdp_classifier_and_assign_taxonomy_no_model_output(self):
        obs = train_rdp_classifier_and_assign_taxonomy(
            self.reference_file, self.taxonomy_file, self.test_trained_input)
        exp = {'X67228': (
            'Bacteria;Proteobacteria;Alphaproteobacteria;Rhizobiales;'
            'Rhizobiaceae;Rhizobium', 1.0
            )}
        self.assertEqual(obs, exp)

# Sample data copied from rdp_classifier-2.0, which is licensed under
# the GPL 2.0 and Copyright 2008 Michigan State University Board of
# Trustees

rdp_training_sequences = """>X67228 Bacteria;Proteobacteria;Alphaproteobacteria;Rhizobiales;Rhizobiaceae;Rhizobium
aacgaacgctggcggcaggcttaacacatgcaagtcgaacgctccgcaaggagagtggcagacgggtgagtaacgcgtgggaatctacccaaccctgcggaatagctctgggaaactggaattaataccgcatacgccctacgggggaaagatttatcggggatggatgagcccgcgttggattagctagttggtggggtaaaggcctaccaaggcgacgatccatagctggtctgagaggatgatcagccacattgggactgagacacggcccaaa
>X73443 Bacteria;Firmicutes;Clostridia;Clostridiales;Clostridiaceae;Clostridium
nnnnnnngagatttgatcctggctcaggatgaacgctggccggccgtgcttacacatgcagtcgaacgaagcgcttaaactggatttcttcggattgaagtttttgctgactgagtggcggacgggtgagtaacgcgtgggtaacctgcctcatacagggggataacagttagaaatgactgctaataccnnataagcgcacagtgctgcatggcacagtgtaaaaactccggtggtatgagatggacccgcgtctgattagctagttggtggggt
>AB004750 Bacteria;Proteobacteria;Gammaproteobacteria;Enterobacteriales;Enterobacteriaceae;Enterobacter
acgctggcggcaggcctaacacatgcaagtcgaacggtagcagaaagaagcttgcttctttgctgacgagtggcggacgggtgagtaatgtctgggaaactgcccgatggagggggataactactggaaacggtagctaataccgcataacgtcttcggaccaaagagggggaccttcgggcctcttgccatcggatgtgcccagatgggattagctagtaggtggggtaacggctcacctaggcgacgatccctagctggtctgagaggatgaccagccacactggaactgagacacggtccagactcctacgggaggcagcagtggggaatattgca
>xxxxxx Bacteria;Proteobacteria;Gammaproteobacteria;Pseudomonadales;Pseudomonadaceae;Pseudomonas
ttgaacgctggcggcaggcctaacacatgcaagtcgagcggcagcannnncttcgggaggctggcgagcggcggacgggtgagtaacgcatgggaacttacccagtagtgggggatagcccggggaaacccggattaataccgcatacgccctgagggggaaagcgggctccggtcgcgctattggatgggcccatgtcggattagttagttggtggggtaatggcctaccaaggcgacgatccgtagctggtctgagaggatgatcagccacaccgggactgagacacggcccggactcctacgggaggcagcagtggggaatattggacaatgggggcaaccctgatccagccatgccg
>AB004748 Bacteria;Proteobacteria;Gammaproteobacteria;Enterobacteriales;Enterobacteriaceae;Enterobacter
acgctggcggcaggcctaacacatgcaagtcgaacggtagcagaaagaagcttgcttctttgctgacgagtggcggacgggtgagtaatgtctgggaaactgcccgatggagggggataactactggaaacggtagctaataccgcataacgtcttcggaccaaagagggggaccttcgggcctcttgccatcggatgtgcccagatgggattagctagtaggtggggtaacggctcacctaggcgacgatccctagctggtctgagaggatgaccagccacactggaactgagacacggtccagactcctacgggaggcagcagtggggaatattgcacaatgggcgcaagcctgatgcagccatgccgcgtgtatgaagaaggccttcgggttg
>AB000278 Bacteria;Proteobacteria;Gammaproteobacteria;Vibrionales;Vibrionaceae;Photobacterium
caggcctaacacatgcaagtcgaacggtaanagattgatagcttgctatcaatgctgacgancggcggacgggtgagtaatgcctgggaatataccctgatgtgggggataactattggaaacgatagctaataccgcataatctcttcggagcaaagagggggaccttcgggcctctcgcgtcaggattagcccaggtgggattagctagttggtggggtaatggctcaccaaggcgacgatccctagctggtctgagaggatgatcagccacactggaactgagacacggtccagactcctacgggaggcagcagtggggaatattgcacaatgggggaaaccctgatgcagccatgccgcgtgta
>AB000390 Bacteria;Proteobacteria;Gammaproteobacteria;Vibrionales;Vibrionaceae;Vibrio
tggctcagattgaacgctggcggcaggcctaacacatgcaagtcgagcggaaacgantnntntgaaccttcggggnacgatnacggcgtcgagcggcggacgggtgagtaatgcctgggaaattgccctgatgtgggggataactattggaaacgatagctaataccgcataatgtctacggaccaaagagggggaccttcgggcctctcgcttcaggatatgcccaggtgggattagctagttggtgaggtaatggctcaccaaggcgacgatccctagctggtctgagaggatgatcagccacactggaactgag
"""

rdp_training_taxonomy = """\
1*Bacteria*0*0*domain
765*Firmicutes*1*1*phylum
766*Clostridia*765*2*class
767*Clostridiales*766*3*order
768*Clostridiaceae*767*4*family
769*Clostridium*768*5*genus
160*Proteobacteria*1*1*phylum
433*Gammaproteobacteria*160*2*class
586*Vibrionales*433*3*order
587*Vibrionaceae*586*4*family
588*Vibrio*587*5*genus
592*Photobacterium*587*5*genus
552*Pseudomonadales*433*3*order
553*Pseudomonadaceae*552*4*family
554*Pseudomonas*553*5*genus
604*Enterobacteriales*433*3*order
605*Enterobacteriaceae*604*4*family
617*Enterobacter*605*5*genus
161*Alphaproteobacteria*160*2*class
260*Rhizobiales*161*3*order
261*Rhizobiaceae*260*4*family
262*Rhizobium*261*5*genus"""

exp_training_header_template = "<trainsetNo>1</trainsetNo><version>version1</version><modversion>cogent</modversion><file>%s</file>\n"

rdp_trained_fasta = """>X67228
aacgaacgctggcggcaggcttaacacatgcaagtcgaacgctccgcaaggagagtggcagacgggtgagtaacgcgtgggaatctacccaaccctgcggaatagctctgggaaactggaattaataccgcatacgccctacgggggaaagatttatcggggatggatgagcccgcgttggattagctagttggtggggtaaaggcctaccaaggcgacgatccatagctggtctgagaggatgatcagccacattgggactgagacacggcccaaa
"""

rdp_sample_fasta = """>X67228 Bacteria;Proteobacteria;Alphaproteobacteria;Rhizobiales;Rhizobiaceae;Rhizobium
aacgaacgctggcggcaggcttaacacatgcaagtcgaacgctccgcaaggagagtggcagacgggtgagtaacgcgtgggaatctacccaaccctgcggaatagctctgggaaactggaattaataccgcatacgccctacgggggaaagatttatcggggatggatgagcccgcgttggattagctagttggtggggtaaaggcctaccaaggcgacgatccatagctggtctgagaggatgatcagccacattgggactgagacacggcccaaa
"""

rdp_sample_classification = """>X67228 reverse=false
Root; 1.0; Bacteria; 1.0; Proteobacteria; 1.0; Alphaproteobacteria; 1.0; Rhizobiales; 1.0; Rhizobiaceae; 1.0; Rhizobium; 0.95; 
"""

rdp_test_fasta = """>AY800210 description field
TTCCGGTTGATCCTGCCGGACCCGACTGCTATCCGGATGCGACTAAGCCATGCTAGTCTAACGGATCTTCGGATCCGTGGCATACCGCTCTGTAACACGTAGATAACCTACCCTGAGGTCGGGGAAACTCCCGGGAAACTGGGCCTAATCCCCGATAGATAATTTGTACTGGAATGTCTTTTTATTGAAACCTCCGAGGCCTCAGGATGGGTCTGCGCCAGATTATGGTCGTAGGTGGGGTAACGGCCCACCTAGCCTTTGATCTGTACCGGACATGAGAGTGTGTGCCGGGAGATGGCCACTGAGACAAGGGGCCAGGCCCTACGGGGCGCAGCAGGCGCGAAAACTTCACAATGCCCGCAAGGGTGATGAGGGTATCCGAGTGCTACCTTAGCCGGTAGCTTTTATTCAGTGTAAATAGCTAGATGAATAAGGGGAGGGCAAGGCTGGTGCCAGCCGCCGCGGTAAAACCAGCTCCCGAGTGGTCGGGATTTTTATTGGGCCTAAAGCGTCCGTAGCCGGGCGTGCAAGTCATTGGTTAAATATCGGGTCTTAAGCCCGAACCTGCTAGTGATACTACACGCCTTGGGACCGGAAGAGGCAAATGGTACGTTGAGGGTAGGGGTGAAATCCTGTAATCCCCAACGGACCACCGGTGGCGAAGCTTGTTCAGTCATGAACAACTCTACACAAGGCGATTTGCTGGGACGGATCCGACGGTGAGGGACGAAACCCAGGGGAGCGAGCGGGATTAGATACCCCGGTAGTCCTGGGCGTAAACGATGCGAACTAGGTGTTGGCGGAGCCACGAGCTCTGTCGGTGCCGAAGCGAAGGCGTTAAGTTCGCCGCCAGGGGAGTACGGCCGCAAGGCTGAAACTTAAAGGAATTGGCGGGGGAGCAC
>EU883771
TGGCGTACGGCTCAGTAACACGTGGATAACTTACCCTTAGGACTGGGATAACTCTGGGAAACTGGGGATAATACTGGATATTAGGCTATGCCTGGAATGGTTTGCCTTTGAAATGTTTTTTTTCGCCTAAGGATAGGTCTGCGGCTGATTAGGTCGTTGGTGGGGTAATGGCCCACCAAGCCGATGATCGGTACGGGTTGTGAGAGCAAGGGCCCGGAGATGGAACCTGAGACAAGGTTCCAGACCCTACGGGGTGCAGCAGGCGCGAAACCTCCGCAATGTACGAAAGTGCGACGGGGGGATCCCAAGTGTTATGCTTTTTTGTATGACTTTTCATTAGTGTAAAAAGCTTTTAGAATAAGAGCTGGGCAAGACCGGTGCCAGCCGCCGCGGTAACACCGGCAGCTCGAGTGGTGACCACTTTTATTGGGCTTAAAGCGTTCGTAGCTTGATTTTTAAGTCTCTTGGGAAATCTCACGGCTTAACTGTGAGGCGTCTAAGAGATACTGGGAATCTAGGGACCGGGAGAGGTAAGAGGTACTTCAGGGGTAGAAGTGAAATTCTGTAATCCTTGAGGGACCACCGATGGCGAAGGCATCTTACCAGAACGGCTTCGACAGTGAGGAACGAAAGCTGGGGGAGCGAACGGGATTAGATACCCCGGTAGTCCCAGCCGTAAACTATGCGCGTTAGGTGTGCCTGTAACTACGAGTTACCGGGGTGCCGAAGTGAAAACGTGAAACGTGCCGCCTGGGAAGTACGGTCGCAAGGCTGAAACTTAAAGGAATTGGCGGGGGAGCACCACAACGGGTGGAGCCTGCGGTTTAATTGGACTCAACGCCGGGCAGCTCACCGGATAGGACAGCGGAATGATAGCCGGGCTGAAGACCTTGCTTGACCAGCTGAGA
>EF503699
AAGAATGGGGATAGCATGCGAGTCACGCCGCAATGTGTGGCATACGGCTCAGTAACACGTAGTCAACATGCCCAGAGGACGTGGACACCTCGGGAAACTGAGGATAAACCGCGATAGGCCACTACTTCTGGAATGAGCCATGACCCAAATCTATATGGCCTTTGGATTGGACTGCGGCCGATCAGGCTGTTGGTGAGGTAATGGCCCACCAAACCTGTAACCGGTACGGGCTTTGAGAGAAGGAGCCCGGAGATGGGCACTGAGACAAGGGCCCAGGCCCTATGGGGCGCAGCAGGCACGAAACCTCTGCAATAGGCGAAAGCTTGACAGGGTTACTCTGAGTGATGCCCGCTAAGGGTATCTTTTGGCACCTCTAAAAATGGTGCAGAATAAGGGGTGGGCAAGTCTGGTGTCAGCCGCCGCGGTAATACCAGCACCCCGAGTTGTCGGGACGATTATTGGGCCTAAAGCATCCGTAGCCTGTTCTGCAAGTCCTCCGTTAAATCCACCCGCTTAACGGATGGGCTGCGGAGGATACTGCAGAGCTAGGAGGCGGGAGAGGCAAACGGTACTCAGTGGGTAGGGGTAAAATCCTTTGATCTACTGAAGACCACCAGTGGTGAAGGCGGTTCGCCAGAACGCGCTCGAACGGTGAGGATGAAAGCTGGGGGAGCAAACCGGAATAGATACCCGAGTAATCCCAACTGTAAACGATGGCAACTCGGGGATGGGTTGGCCTCCAACCAACCCCATGGCCGCAGGGAAGCCGTTTAGCTCTCCCGCCTGGGGAATACGGTCCGCAGAATTGAACCTTAAAGGAATTTGGCGGGGAACCCCCACAAGGGGGAAAACCGTGCGGTTCAATTGGAATCCACCCCCCGGAAACTTTACCCGGGCGCG
>random_seq
AAGCTCCGTCGCGTGAGCTAAAAACCATGCTGACTTATGAGACCTAAAAGCGATGCGCCGACCTGACGATGCTCTGTTCAGTTTCATCACGATCACCGGTAGTCAGGGTACCCTCCAGACCGCGCATAGTGACTATGTTCCCGCACCTGTATATGTAATTCCCATTATACGTCTACGTTATGTAGTAAAGTTGCTCACGCCAGGCACAGTTTGTCTTGATACATAGGGTAGCTTAAGTCCCGTCCATTTCACCGCGATTGTAATAGACGAATCAGCAGTGGTGCAATCAAGTCCCAACAGTTATATTTCAAAAATCTTCCGATAGTCGTGGGCGAAGTTGTCAACCTACCTACCATGGCTATAAGGCCCAGTTTACTTCAGTTGAACGTGACGGTAACCCTACTGAGTGCACGATACCTGCTCAACAACGGCCCAAAACCCGTGCGACACATTGGGCACTACAATAATCTTAGAGGACCATGGATCTGGTGGGTGGACTGAAGCATATCCCAAAAGTGTCGTGAGTCCGTTATGCAATTGACTGAAACAGCCGTACCAGAGTTCGGATGACCTCTGGGTTGCTGCGGTACACACCCGGGTGCGGCTTCTGAAATAGAAAAGACTAAGCATCGGCCGCCTCACACGCCAC
>DQ260310
GATACCCCCGGAAACTGGGGATTATACCGGATATGTGGGGCTGCCTGGAATGGTACCTCATTGAAATGCTCCCGCGCCTAAAGATGGATCTGCCGCAGAATAAGTAGTTTGCGGGGTAAATGGCCACCCAGCCAGTAATCCGTACCGGTTGTGAAAACCAGAACCCCGAGATGGAAACTGAAACAAAGGTTCAAGGCCTACCGGGCACAACAAGCGCCAAAACTCCGCCATGCGAGCCATCGCGACGGGGGAAAACCAAGTACCACTCCTAACGGGGTGGTTTTTCCGAAGTGGAAAAAGCCTCCAGGAATAAGAACCTGGGCCAGAACCGTGGCCAGCCGCCGCCGTTACACCCGCCAGCTCGAGTTGTTGGCCGGTTTTATTGGGGCCTAAAGCCGGTCCGTAGCCCGTTTTGATAAGGTCTCTCTGGTGAAATTCTACAGCTTAACCTGTGGGAATTGCTGGAGGATACTATTCAAGCTTGAAGCCGGGAGAAGCCTGGAAGTACTCCCGGGGGTAAGGGGTGAAATTCTATTATCCCCGGAAGACCAACTGGTGCCGAAGCGGTCCAGCCTGGAACCGAACTTGACCGTGAGTTACGAAAAGCCAAGGGGCGCGGACCGGAATAAAATAACCAGGGTAGTCCTGGCCGTAAACGATGTGAACTTGGTGGTGGGAATGGCTTCGAACTGCCCAATTGCCGAAAGGAAGCTGTAAATTCACCCGCCTTGGAAGTACGGTCGCAAGACTGGAACCTAAAAGGAATTGGCGGGGGGACACCACAACGCGTGGAGCCTGGCGGTTTTATTGGGATTCCACGCAGACATCTCACTCAGGGGCGACAGCAGAAATGATGGGCAGGTTGATGACCTTGCTTGACAAGCTGAAAAGGAGGTGCAT
>EF503697
TAAAATGACTAGCCTGCGAGTCACGCCGTAAGGCGTGGCATACAGGCTCAGTAACACGTAGTCAACATGCCCAAAGGACGTGGATAACCTCGGGAAACTGAGGATAAACCGCGATAGGCCAAGGTTTCTGGAATGAGCTATGGCCGAAATCTATATGGCCTTTGGATTGGACTGCGGCCGATCAGGCTGTTGGTGAGGTAATGGCCCACCAAACCTGTAACCGGTACGGGCTTTGAGAGAAGTAGCCCGGAGATGGGCACTGAGACAAGGGCCCAGGCCCTATGGGGCGCAGCAGGCGCGAAACCTCTGCAATAGGCGAAAGCCTGACAGGGTTACTCTGAGTGATGCCCGCTAAGGGTATCTTTTGGCACCTCTAAAAATGGTGCAGAATAAGGGGTGGGCAAGTCTGGTGTCAGCCGCCGCGGTAATACCAGCACCCCGAGTTGTCGGGACGATTATTGGGCCTAAAGCATCCGTAGCCTGTTCTGCAAGTCCTCCGTTAAATCCACCTGCTCAACGGATGGGCTGCGGAGGATACCGCAGAGCTAGGAGGCGGGAGAGGCAAACGGTACTCAGTGGGTAGGGGTAAAATCCATTGATCTACTGAAGACCACCAGTGGCGAAGGCGGTTTGCCAGAACGCGCTCGACGGTGAGGGATGAAAGCTGGGGGAGCAAACCGGATTAGATACCCGGGGTAGTCCCAGCTGTAAACGGATGCAGACTCGGGTGATGGGGTTGGCTTCCGGCCCAACCCCAATTGCCCCCAGGCGAAGCCCGTTAAGATCTTGCCGCCCTGTCAGATGTCAGGGCCGCCAATACTCGAAACCTTAAAAGGAAATTGGGCGCGGGAAAAGTCACCAAAAGGGGGTTGAAACCCTGCGGGTTATATATTGTAAACC
>short_seq
TAAAATGACTAGCCTGCGAGTCAC
"""

rdp_expected_out = {
    'AY800210 description field': 'Archaea;Euryarchaeota',
    'EU883771': 'Archaea;Euryarchaeota;Methanomicrobia;Methanomicrobiales;Methanomicrobiaceae;Methanomicrobium',
    'EF503699': 'Archaea;Crenarchaeota;Thermoprotei',
    'random_seq': 'Bacteria',
    'DQ260310': 'Archaea;Euryarchaeota;Methanobacteria;Methanobacteriales;Methanobacteriaceae;Methanosphaera',
    'EF503697': 'Archaea;Crenarchaeota;Thermoprotei',
    'short_seq': 'Unassignable',
    }

if __name__ == '__main__':
    main()




#!/usr/bin/env python
from warnings import catch_warnings, simplefilter, warn as _warn

__author__ = "Gavin Huttley"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "Jai Ram Rideout"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

def deprecated(_type, old, new, version, stack_level=2):
    """a convenience function for deprecating classes, functions, arguments.
    
    Arguments:
        - _type should be one of class, method, function, argument
        - old, new: the old and new names
        - version: the version by which support for the old name will be
          discontinued
        - stack_level: as per warnings.warn"""
    msg = "use %s %s instead of %s, support discontinued in version %s" % \
        (_type, new, old, version)

    # DeprecationWarnings are ignored by default in python 2.7, so temporarily
    # force them to be handled.
    with catch_warnings():
        simplefilter("always")
        _warn(msg, DeprecationWarning, stack_level)

def discontinued(_type, name, version, stack_level=2):
    """convenience func to warn about discontinued attributes
    Arguments:
        - _type should be one of class, method, function, argument
        - name: the attributes name
        - version: the version by which support for the old name will be
          discontinued
        - stack_level: as per warnings.warn"""
    msg = "%s %s is discontinued, support will be stopped in version %s" %\
        (_type, name, version)

    with catch_warnings():
        simplefilter("always")
        _warn(msg, DeprecationWarning, stack_level)



#!/usr/bin/env python
"""Package cogent.format: provides modules for writing specific file formats.

Currently provides:
    mage: writers for the MAGE 3D visualization program
    xml:  xml base class
    file: general functions to read and write files
"""
__all__ = ['alignment', 'clustal', 'fasta', 'mage', 'motif', 'nexus', 
           'pdb_color', 'pdb', 'phylip', 'rna_struct', 'stockholm', 'structure', 
           'table', 'text_tree', 'xyzrn']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jeremy Widmann", "Gavin Huttley", "Matthew Wakefield",
               "Rob Knight", "Sandra Smit", "Peter Maxwell", "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"



import os

from cogent import DNA
from cogent.util.unit_test import TestCase, main
from cogent.db.ensembl.host import HostAccount, get_ensembl_account
from cogent.db.ensembl.genome import Genome
from cogent.db.ensembl.assembly import CoordSystem, Coordinate, get_coord_conversion
from cogent.db.ensembl.feature_level import FeatureCoordLevels

__author__ = "Gavin Huttley, Hua Ying"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "hua Ying"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "Gavin.Huttley@anu.edu.au"
__status__ = "alpha"

Release = 81

if 'ENSEMBL_ACCOUNT' in os.environ:
    args = os.environ['ENSEMBL_ACCOUNT'].split()
    host, username, password = args[0:3]
    kwargs = {}
    if len(args) > 3:
        kwargs['port'] = int(args[3])
    account = HostAccount(host, username, password, **kwargs)
else:
    account = get_ensembl_account(release=Release)


class TestFeatureCoordLevels(TestCase):
    def setUp(self):
        self.chicken = Genome(Species='chicken', Release=Release,
                            account=account)
    
    def test_feature_levels(self):
        ChickenFeatureLevels = FeatureCoordLevels('chicken')
        chicken_feature_levels = ChickenFeatureLevels(
                    feature_types=['gene', 'cpg', 'est'],
                    core_db=self.chicken.CoreDb,
                    otherfeature_db=self.chicken.OtherFeaturesDb)
        self.assertEquals(chicken_feature_levels['repeat'].levels,
                                ['chromosome', 'scaffold'])
        self.assertEquals(set(chicken_feature_levels['cpg'].levels),
                            set(['chromosome', 'scaffold']))
    
    def test_repeat(self):
        # use chicken genome as it need to do conversion
        # chicken coordinate correspondent toRefSeq human IL2A region
        coord = dict(CoordName=9, Start=21727352, End=21729141)
        region = self.chicken.getRegion(**coord)
        # repeat is recorded at contig level, strand is 0
        repeats = region.getFeatures(feature_types = 'repeat')
        expect = [("9", 21727499, 21727527), ("9", 21728009, 21728018),
                  ("9", 21728169, 21728178)]
        obs = []
        for repeat in repeats:
            loc = repeat.Location
            obs.append((str(loc.CoordName), loc.Start, loc.End))
        self.assertEquals(set(obs), set(expect))
    
    def test_cpg(self):
        # contain 3 CpG island recorded at chromosome level
        coord1 = dict(CoordName=26, Start=105184, End=184346)
        cpgs1 = self.chicken.getFeatures(feature_types='cpg', **coord1)
        exp = [("26", 112153, 113139), ("26", 134125, 135050),
               ("26", 178899, 180227)]
        obs = []
        for cpg in cpgs1:
            loc = cpg.Location
            obs.append((str(loc.CoordName), loc.Start, loc.End))
        self.assertEquals(set(obs), set(exp))
        
        # test cpg features record at scaffold level:
        coord2 = dict(CoordName='JH376196.1', Start=1, End=14640)
        cpgs2 = self.chicken.getFeatures(feature_types='cpg', **coord2)
        self.assertEquals(len(list(cpgs2)), 3)
    

if __name__ == '__main__':
    main()



#!/usr/bin/env python

__all__ = ['aaindex',
           'agilent_microarray',
           'blast',
           'bpseq',
           'carnac',
           'cigar',
           'clustal',
           'cmfinder',
           'column',
           'comrna',
           'consan',
           'contrafold',
           'cove',
           'ct',
           'cut',
           'cutg',
           'dialign',
           'dynalign',
           'ebi',
           'fasta',
           'fastq',
           'foldalign',
           'gbseq',
           'gcg',
           'genbank',
           'gff',
           'illumina_sequence',
           'ilm',
           'knetfold',
           'locuslink',
           'macsim',
           'mage',
           'meme',
           'mfold',
           'ncbi_taxonomy',
           'newick',
           'nexus',
           'nupack',
           'paml',
           'paml_matrix',
           'pdb',
           'pfold',
           'phylip',
           'pknotsrg',
           'rdb',
           'record',
           'record_finder',
           'rfam',
           'rna_fold',
           'rnaalifold',
           'rnaforester',
           'rnashapes',
           'rnaview',
           'sequence',
           'sfold',
           'sprinzl',
           'table',
           'tinyseq',
           'tree',
           'tree_xml',
           'unafold',
           'unigene']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "Peter Maxwell", "Rob Knight",
                    "Catherine Lozupone", "Jeremy Widmann", "Matthew Wakefield",
                    "Sandra Smit", "Greg Caporaso", "Zongzhi Liu",
                    "Micah Hamady", "Jason Carnes", "Raymond Sammut",
                    "Hua Ying", "Andrew Butterfield", "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"



"""Contains functions to annotate macromolecular entities."""

from cogent.core.entity import HIERARCHY

__author__ = "Marcin Cieslik"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Marcin Cieslik"
__email__ = "mpc4p@virginia.edu"
__status__ = "Development"

def xtradata(data, entity):
    """Annotates an entity with data from a ``{full_id:data}`` dictionary. The
    ``data`` should also be a dictionary.
    
    Arguments:
        - data: a dictionary, which is a mapping of full_id's (keys) and data
                dictionaries.
        - entity: top-level entity, which contains the entities which will hold 
                  the data."""
    for full_id, data in data.iteritems():
        sub_entity = entity
        strip_full_id = [i for i in full_id if i is not None]
        for short_id in strip_full_id:
            sub_entity = sub_entity[(short_id,)]
        sub_entity.xtra.update(data)




#!/usr/bin/env python

__all__ = ['energy_params', 'molecular_weight', 'protein_properties', \
           'ligand_properties', 'nucleic_properties']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Amanda Birmingham", "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"



#!/usr/bin/env python
"""Unit tests for locuslink-specific classes
"""
from cogent.parse.locuslink import ll_start,LLFinder,pipes,first_pipe,commas, \
    _read_accession, _read_rell, _read_accnum, \
    _read_map, _read_sts, _read_comp, _read_grif, _read_pmid, _read_go, \
    _read_extannot, _read_cdd, _read_contig, LocusLink, LinesToLocusLink
from cogent.util.unit_test import TestCase, main

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

class locuslinkTests(TestCase):
    """Tests toplevel functions."""

    def test_read_accession(self):
        """_read_accession should perform correct conversions"""
        self.assertEqual(_read_accession('NP_035835|6755985|na\n'), \
            {'Accession':'NP_035835','Gi':'6755985','Strain':'na'})
        #check that it ignores additional fields
        self.assertEqual(_read_accession('NG_002740|30172554|na|1|1315\n'), \
            {'Accession':'NG_002740','Gi':'30172554','Strain':'na'})

    def test_read_rell(self):
        """_read_rell should perform correct conversions"""
        self.assertEqual(_read_rell(\
            'related mRNA|AK090391|n|NM_153775--AK090391\n'), \
            {'Description':'related mRNA','Id':'AK090391','IdType':'n',\
            'Printable':'NM_153775--AK090391'})
 
    def test_read_accnum(self):
        """_read_accnum should perform correct conversions"""
        self.assertEqual(_read_accnum('NG_002740|30172554|na|1|1315\n'), \
            {'Accession':'NG_002740','Gi':'30172554','Strain':'na',\
            'Start':'1','End':'1315'})

    def test_read_map(self):
        """_read_map should perform correct conversions"""
        self.assertEqual(_read_map('10 C1|RefSeq|C|\n'), \
            {'Location':'10 C1', 'Source':'RefSeq','Type':'C'})

    def test_read_sts(self):
        """_read_sts should perform correct conversions"""
        self.assertEqual(_read_sts('RH35858|2|37920|na|seq_map|epcr\n'), \
            {'Name':'RH35858','Chromosome':'2','StsId':'37920', 'Segment':'na',\
            'SequenceKnown':'seq_map', 'Evidence':'epcr'})
    
    def test_read_cdd(self):
        """_read_cdd should perform correct conversions"""
        self.assertEqual(_read_cdd(\
        'Immunoglobulin C-2 Type|smart00408|103|na|4.388540e+01\n'),
            {'Name':'Immunoglobulin C-2 Type','Key':'smart00408',\
            'Score':'103', 'EValue':'na', 'BitScore':'4.388540e+01'})

    def test_read_comp(self):
        """_read_comp should perform correct conversions"""
        self.assertEqual(_read_comp(\
            '10090|Map2k6|11|11  cM|26399|17|MAP2K6|ncbi_mgd\n'), \
            {'TaxonId':'10090', 'Symbol':'Map2k6', 'Chromosome':'11', \
            'Position':'11  cM', 'LocusId':'26399', 'ChromosomeSelf':'17', \
            'SymbolSelf':'MAP2K6','MapName':'ncbi_mgd'})

    def test_read_grif(self):
        """_read_grif should perform correct conversions"""
        self.assertEqual(_read_grif('12037672|interaction with pRb\n'), \
            {'PubMedId':'12037672', 'Description':'interaction with pRb'})

    def test_read_pmid(self):
        """_read_pmid should perform correct conversions"""
        self.assertEqual(_read_pmid('12875969,12817023,12743034\n'), \
            ['12875969','12817023','12743034'])

    def test_read_go(self):
        """_read_go should perform correct conversions"""
        self.assertEqual(_read_go(\
            'molecular function|zinc ion binding|IEA|GO:0008270|GOA|na\n'), \
            {'Category':'molecular function', 'Term':'zinc ion binding',\
            'EvidenceCode':'IEA','GoId':'GO:0008270','Source':'GOA', \
            'PubMedId':'na'})

    def test_read_extannot(self):
        """_read_extannot should perform correct conversions"""
        self.assertEqual(_read_extannot(\
        'cellular role|Pol II transcription|NR|Proteome|8760285\n'), \
        {'Category':'cellular role','Term':'Pol II transcription',\
        'EvidenceCode':'NR', 'Source':'Proteome', 'PubMedId':'8760285'})
            
    def test_read_contig(self):
        """_read_contig should perform correct conversions"""
        self.assertEqual(_read_contig(\
        'NT_011109.15|29800594|na|31124734|31133047|-|19|reference\n'),\
        {'Accession':'NT_011109.15','Gi':'29800594','Strain':'na',\
        'From':'31124734','To':'31133047','Orientation':'-',\
        'Chromosome':'19','Assembly':'reference'})

    def test_LinesToLocusLink(self):
        """LinesToLocusLink should give expected results on sample data"""
        fake_file = \
""">>1
LOCUSID: 1
LOCUS_CONFIRMED: yes
LOCUS_TYPE: gene with protein product, function known or inferred
ORGANISM: Homo sapiens
STATUS: REVIEWED
NM: NM_130786|21071029|na
NP: NP_570602|21071030
CDD: Immunoglobulin C-2 Type|smart00408|103|na|4.388540e+01
PRODUCT: alpha 1B-glycoprotein
ASSEMBLY: AF414429,AK055885,AK056201
CONTIG: NT_011109.15|29800594|na|31124734|31133047|-|19|reference
EVID: supported by alignment with mRNA
XM: NM_130786|21071029|na
XP: NP_570602|21071030|na
ACCNUM: AC010642|9929687|na|43581|41119
TYPE: g
ACCNUM: AF414429|15778555|na|na|na
TYPE: m
PROT: AAL07469|15778556
ACCNUM: AK055885|16550723|na|na|na
TYPE: m
ACCNUM: AK056201|16551539|na|na|na
TYPE: m
ACCNUM: BC035719|23273475|na|na|na
TYPE: m
PROT: AAH35719|23273476
ACCNUM: none|na|na|na|na
TYPE: p
PROT: P04217|23503038
OFFICIAL_SYMBOL: A1BG
OFFICIAL_GENE_NAME: alpha-1-B glycoprotein
ALIAS_SYMBOL: A1B
ALIAS_SYMBOL: ABG
ALIAS_SYMBOL: GAB
PREFERRED_PRODUCT: alpha 1B-glycoprotein
SUMMARY: Summary: The protein encoded by this gene is a plasma glycoprotein of unknown function. The protein shows sequence similarity to the variable regions of some immunoglobulin supergene family member proteins.
CHR: 19
STS: RH65092|-|10673|na|na|epcr
STS: WI-16009|-|52209|na|na|epcr
STS: G59506|-|136670|na|na|epcr
COMP: 10090|A1bg|na|na|117586|19|A1BG|ncbi_mgd
COMP: 10090|A1bg|7|7  cM|117586|19|A1BG|ncbi_mgd
BUTTON: unigene.gif
LINK: http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=Hs&CID=390608
UNIGENE: Hs.390608
OMIM: 138670
MAP: 19q13.4|RefSeq|C|
MAPLINK: default_human_gene|A1BG
BUTTON: snp.gif
LINK: http://www.ncbi.nlm.nih.gov/SNP/snp_ref.cgi?locusId=1
BUTTON: homol.gif
LINK: http://www.ncbi.nlm.nih.gov/HomoloGene/homolquery.cgi?TEXT=1[loc]&TAXID=9606
BUTTON: ensembl.gif
LINK: http://www.ensembl.org/Homo_sapiens/contigview?geneid=NM_130786
BUTTON: ucsc.gif
LINK: http://genome.ucsc.edu/cgi-bin/hgTracks?org=human&position=NM_130786
BUTTON: mgc.gif
LINK: http://mgc.nci.nih.gov/Genes/GeneInfo?ORG=Hs&CID=390608
PMID: 12477932,8889549,3458201,2591067
GO: molecular function|molecular_function unknown|ND|GO:0005554|GOA|3458201
GO: biological process|biological_process unknown|ND|GO:0000004|GOA|na
GO: cellular component|extracellular|IDA|GO:0005576|GOA|3458201
>>386590
LOCUSID: 386590
LOCUS_CONFIRMED: yes
LOCUS_TYPE: gene with protein product, function known or inferred
ORGANISM: Danio rerio
ACCNUM: AF510108|31323727|na|na|na
TYPE: m
PROT: AAP47138|31323728
OFFICIAL_SYMBOL: tra1
OFFICIAL_GENE_NAME: tumor rejection antigen (gp96) 1
BUTTON: zfin.gif
LINK: http://zfin.org/cgi-bin/ZFIN_jump?record=ZDB-GENE-031002-1
PMID: 14499652"""

        records = list(LLFinder(fake_file.split('\n')))
        self.assertEqual(len(records), 2)
        first, second = map(LinesToLocusLink, records)

        #test the second one first, since it's shorter
        self.assertEqual(second.LOCUSID, 386590)
        self.assertEqual(second.LOCUS_CONFIRMED, 'yes')
        self.assertEqual(second.LOCUS_TYPE, \
            'gene with protein product, function known or inferred')
        self.assertEqual(second.ORGANISM, 'Danio rerio')
        self.assertEqual(second.ACCNUM, [{'Accession':'AF510108', \
            'Gi':'31323727', 'Strain':'na','Start':'na','End':'na'}])
        self.assertEqual(second.TYPE, ['m'])
        self.assertEqual(second.PROT, \
            [{'Accession':'AAP47138','Gi':'31323728'}])
        self.assertEqual(second.OFFICIAL_SYMBOL, 'tra1')
        self.assertEqual(second.OFFICIAL_GENE_NAME, \
            'tumor rejection antigen (gp96) 1')
        self.assertEqual(second.BUTTON, ['zfin.gif'])
        self.assertEqual(second.LINK, \
            ['http://zfin.org/cgi-bin/ZFIN_jump?record=ZDB-GENE-031002-1'])
        self.assertEqual(second.PMID, ['14499652'])

        #now for the annoying test on the longer record
        self.assertEqual(first.LOCUSID, 1)
        self.assertEqual(first.LOCUS_CONFIRMED, 'yes')
        self.assertEqual(first.ORGANISM, 'Homo sapiens')
        self.assertEqual(first.LOCUS_TYPE, \
            'gene with protein product, function known or inferred')
        self.assertEqual(first.STATUS, 'REVIEWED')
        self.assertEqual(first.NM, [{'Accession':'NM_130786','Gi':'21071029', \
            'Strain':'na'}])
        self.assertEqual(first.NP, [{'Accession':'NP_570602','Gi':'21071030'}])
        self.assertEqual(first.CDD, [{'Name':'Immunoglobulin C-2 Type',\
            'Key':'smart00408','Score':'103', 'EValue':'na',\
            'BitScore':'4.388540e+01'}])
        self.assertEqual(first.PRODUCT, ['alpha 1B-glycoprotein'])
        self.assertEqual(first.ASSEMBLY, [['AF414429','AK055885','AK056201']])
        self.assertEqual(first.CONTIG, [{'Accession':'NT_011109.15',\
        'Gi':'29800594','Strain':'na', 'From':'31124734','To':'31133047',\
        'Orientation':'-','Chromosome':'19','Assembly':'reference'}])
        self.assertEqual(first.EVID, ['supported by alignment with mRNA'])
        self.assertEqual(first.XM, [{'Accession':'NM_130786', 'Gi':'21071029', \
            'Strain':'na'}])
        self.assertEqual(first.XP, [{'Accession':'NP_570602', 'Gi':'21071030', \
            'Strain':'na'}])
        self.assertEqual(first.ACCNUM, [ \
            {'Accession':'AC010642','Gi':'9929687','Strain':'na',\
            'Start':'43581', 'End':'41119'},
            {'Accession':'AF414429','Gi':'15778555','Strain':'na',\
            'Start':'na', 'End':'na'},
            {'Accession':'AK055885','Gi':'16550723','Strain':'na',\
            'Start':'na', 'End':'na'},
            {'Accession':'AK056201','Gi':'16551539','Strain':'na',\
            'Start':'na', 'End':'na'},
            {'Accession':'BC035719','Gi':'23273475','Strain':'na',\
            'Start':'na', 'End':'na'},
            {'Accession':'none','Gi':'na','Strain':'na',\
            'Start':'na', 'End':'na'},
            ])
        self.assertEqual(first.TYPE, ['g','m','m','m','m','p'])
        self.assertEqual(first.PROT, [ \
            {'Accession':'AAL07469', 'Gi':'15778556'},
            {'Accession':'AAH35719', 'Gi':'23273476'},
            {'Accession':'P04217', 'Gi':'23503038'},
        ])
        self.assertEqual(first.OFFICIAL_SYMBOL, 'A1BG')
        self.assertEqual(first.OFFICIAL_GENE_NAME, 'alpha-1-B glycoprotein')
        self.assertEqual(first.ALIAS_SYMBOL, ['A1B','ABG','GAB'])
        self.assertEqual(first.PREFERRED_PRODUCT, ['alpha 1B-glycoprotein'])
        self.assertEqual(first.SUMMARY, ["""Summary: The protein encoded by this gene is a plasma glycoprotein of unknown function. The protein shows sequence similarity to the variable regions of some immunoglobulin supergene family member proteins."""])
        self.assertEqual(first.CHR, ['19'])
        self.assertEqual(first.STS, [
        {'Name':'RH65092','Chromosome':'-','StsId':'10673','Segment':'na',\
            'SequenceKnown':'na','Evidence':'epcr'},
        {'Name':'WI-16009','Chromosome':'-','StsId':'52209','Segment':'na',\
            'SequenceKnown':'na','Evidence':'epcr'},
        {'Name':'G59506','Chromosome':'-','StsId':'136670','Segment':'na',\
            'SequenceKnown':'na','Evidence':'epcr'},
        ])
        self.assertEqual(first.COMP, [
        {'TaxonId':'10090','Symbol':'A1bg','Chromosome':'na','Position':'na',\
           'LocusId':'117586', 'ChromosomeSelf':'19','SymbolSelf':'A1BG',\
           'MapName':'ncbi_mgd'},
        {'TaxonId':'10090','Symbol':'A1bg','Chromosome':'7','Position':'7  cM',\
           'LocusId':'117586', 'ChromosomeSelf':'19','SymbolSelf':'A1BG',\
           'MapName':'ncbi_mgd'},
        ])
        self.assertEqual(first.BUTTON, ['unigene.gif','snp.gif','homol.gif', \
            'ensembl.gif', 'ucsc.gif', 'mgc.gif'])
        self.assertEqual(first.LINK, [ \
        'http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=Hs&CID=390608',
        'http://www.ncbi.nlm.nih.gov/SNP/snp_ref.cgi?locusId=1',
        'http://www.ncbi.nlm.nih.gov/HomoloGene/homolquery.cgi?TEXT=1[loc]&TAXID=9606',
        'http://www.ensembl.org/Homo_sapiens/contigview?geneid=NM_130786',
        'http://genome.ucsc.edu/cgi-bin/hgTracks?org=human&position=NM_130786',
        'http://mgc.nci.nih.gov/Genes/GeneInfo?ORG=Hs&CID=390608',
        ])
        self.assertEqual(first.UNIGENE, ['Hs.390608'])
        self.assertEqual(first.OMIM, ['138670'])
        self.assertEqual(first.MAP, [{'Location':'19q13.4','Source':'RefSeq',\
            'Type':'C'}])
        self.assertEqual(first.MAPLINK, ['default_human_gene|A1BG'])
        self.assertEqual(first.PMID, ['12477932','8889549','3458201','2591067'])
        self.assertEqual(first.GO, [ \
        {'Category':'molecular function','Term':'molecular_function unknown',\
        'EvidenceCode':'ND','GoId':'GO:0005554','Source':'GOA',\
        'PubMedId':'3458201'},
        {'Category':'biological process','Term':'biological_process unknown',\
        'EvidenceCode':'ND','GoId':'GO:0000004','Source':'GOA',\
        'PubMedId':'na'},
        {'Category':'cellular component','Term':'extracellular',\
        'EvidenceCode':'IDA','GoId':'GO:0005576','Source':'GOA',\
        'PubMedId':'3458201'},
        ])

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""For use with hmmer3"""

from cogent.parse.hmmscan import MinimalTbloutParser, RichTbloutParser
from cogent.util.unit_test import TestCase, main

__author__ = "Daniel McDonald"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Daniel McDonald"] 
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Daniel McDonald"
__email__ = "daniel.mcdonald@colorado.edu"
__status__ = "Prototype"

class hmmscanTests(TestCase):
    def setUp(self):
        pass
    def test_MinimalTbloutParser(self):
        """MinimalTbloutParser tests"""
        exp1 = ['Sulfotransfer_3','PF13469.1','NC_008340.4664','-','1.7e-14',
                '43.6','0.0','3.4e-14','42.6','0.0','1.5','1','0','0','1','1',
                '1','1','Sulfotransferase family']
        exp2 = ['UreD', 'PF01774.12', 'NC_008340.6164', '-', '4e-62', '198.0', 
                '0.0', '4.9e-62', '197.7', '0.0', '1.1', '1', '0', '0', '1', 
                '1', '1', '1', 'UreD urease accessory protein']
        exp3 = ['UreF', 'PF01730.11', 'NC_008340.6172', '-', '1.9e-51', 
                '162.7', '10.3', '2.8e-51', '162.1', '7.1', '1.3', '1', '0', 
                '0', '1', '1', '1', '1', 'UreF']

        parser = MinimalTbloutParser(tblout)
        self.assertEqual(parser.next(), exp1)
        self.assertEqual(parser.next(), exp2)
        self.assertEqual(parser.next(), exp3)
        self.assertRaises(StopIteration, parser.next)

    def test_RichTbloutParser(self):
        """More useful returns"""
        headers = ["target name", "target accession", "query name",
                   "query accession","full seq E-value", "full seq score", 
                   "full seq bias","best 1 domain E-value", 
                   "best 1 domain score", "best 1 domain bias", "exp","reg",
                   "clu","ov","env","dom","rep","inc","description of target"]
        types = [str,str,str,str,float,float,float,float,float,float,float,int,
                 int,int,int,int,int,int,str]
        expbase1 = ['Sulfotransfer_3','PF13469.1','NC_008340.4664','-','1.7e-14',
                '43.6','0.0','3.4e-14','42.6','0.0','1.5','1','0','0','1','1',
                '1','1','Sulfotransferase family']
        expbase2 = ['UreD', 'PF01774.12', 'NC_008340.6164', '-', '4e-62', 
                    '198.0', '0.0', '4.9e-62', '197.7', '0.0', '1.1', '1', '0', 
                    '0', '1', '1', '1', '1', 'UreD urease accessory protein']
        expbase3 = ['UreF', 'PF01730.11', 'NC_008340.6172', '-', '1.9e-51', 
                '162.7', '10.3', '2.8e-51', '162.1', '7.1', '1.3', '1', '0', 
                '0', '1', '1', '1', '1', 'UreF']
       
        exp1 = dict([(h,t(e)) for h,t,e in zip(headers,types,expbase1)])
        exp2 = dict([(h,t(e)) for h,t,e in zip(headers,types,expbase2)])
        exp3 = dict([(h,t(e)) for h,t,e in zip(headers,types,expbase3)])

        parser = RichTbloutParser(tblout)
        self.assertEqual(parser.next(), exp1)
        self.assertEqual(parser.next(), exp2)
        self.assertEqual(parser.next(), exp3)
        self.assertRaises(StopIteration, parser.next)

tblout = """#                                                               --- full sequence ---- --- best 1 domain ---- --- domain number estimation ----
# target name        accession  query name           accession    E-value  score  bias   E-value  score  bias   exp reg clu  ov env dom rep inc description of target
#------------------- ---------- -------------------- ---------- --------- ------ ----- --------- ------ -----   --- --- --- --- --- --- --- --- ---------------------
Sulfotransfer_3      PF13469.1  NC_008340.4664       -            1.7e-14   43.6   0.0   3.4e-14   42.6   0.0   1.5   1   0   0   1   1   1   1 Sulfotransferase family
UreD                 PF01774.12 NC_008340.6164       -              4e-62  198.0   0.0   4.9e-62  197.7   0.0   1.1   1   0   0   1   1   1   1 UreD urease accessory protein
UreF                 PF01730.11 NC_008340.6172       -            1.9e-51  162.7  10.3   2.8e-51  162.1   7.1   1.3   1   0   0   1   1   1   1 UreF""".splitlines()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
""" Provides codon usage bias measurements, the Codon Adaptation Index (CAI).

Codon bias patterns for a gene have been shown to coorelate with translation
levels and the accuracy of translation.  A CAI is derived from codon preference
statistics.  This CAI evaluation measures the extent to which codon usage in a
particular gene matches the usage pattern of a highly expressed gene set. Three
models have been implemented here, each as a function of either an arithmetic
or geometric mean to test multiplicative versus additive fitness:
1) Assumes each gene selects codons and amino acids to maximize translation
   rate.  Gene length may also be selected.
2) Assumes each gene selects codons to maximize translation rate, but not amino
   amino acids.  Assumes the absolute translation rate, independent of amino
   acid frequency, is what's important (Sharp and Li, 1987).
3) Same as (2) above, but assumes the per codon translation rate is what's
   important (Eyre-Walker, 1996).

Note:
-   Frequencies for single codon families (Met and Trp) are omitted from the
    calculations since they can not exhibit bias.
-   Stop codon frequencies are not omitted.
-   The Bulmer (1988) correction is used to prevent small normalized
    frequencies from creating a bias.
"""
from __future__ import division
from numpy import log , exp
from cogent.core.genetic_code import GeneticCodes

__author__ = "Michael Eaton"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Michael Eaton", "Rob Knight", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

# Default cutoff for small codon frequencies to avoid bias
FREQUENCY_MIN = 1.0e-3
# Default Standard Genetic Code NCBI number
SGC = 1
# Default set of codons -- note: need to be RNA for compatibility with
#the standard CodonUsage object.
bases = 'UGAC'
cu = dict.fromkeys([i+j+k for i in bases for j in bases for k in bases], 0.0)

def as_rna(s): 
    """Converts string to uppercase RNA"""
    return s.upper().replace('T','U')

def synonyms_to_rna(syn):
    """Converts values in synonyms dict to uppercase RNA"""
    result = {}
    for k, v in syn.items():
        result[k] = map(as_rna, v)
    return result

def get_synonyms(genetic_code=SGC, singles_removed=True, stops_removed=True):
    """Gets synonymous codon blocks as dict, keyed by amino acid."""
    GC = GeneticCodes[genetic_code]
    synonyms = GC.Synonyms.copy()   #don't modify original
    if stops_removed:
        if '*' in synonyms:
            del synonyms['*']
    if singles_removed:
        for aa, family in synonyms.items():
            if len(family) < 2: #delete any empty ones as well
                del synonyms[aa]
    return synonyms_to_rna(synonyms)

def sum_codon_freqs(freqs):
    """Sums a set of individual codon freqs, assuming dicts.

    Omits invalid keys.
    """
    result = cu.copy()
    for f in freqs:
        for k, v in f.items():
            if k in cu:
                cu[k] += v
    return cu

def norm_to_max(vals):
    """Normalizes items in vals relative to max val: returns copy."""
    best = max(vals)
    return [i/best for i in vals]

def arithmetic_mean(vals, freqs=None):
    """Returns arithmetic mean of vals."""
    if freqs is None:
        return sum(vals)/float(len(vals))
    else:
        return sum([v*i for v, i in zip(vals, freqs)])/sum(freqs)

def geometric_mean(vals, freqs=None):
    """Returns geometric mean of vals."""
    if freqs is None:
        return exp(log(vals).sum()/float(len(vals)))
    else:
        return exp(sum([v*i for v,i in zip(log(vals), freqs)])/sum(freqs))

def codon_adaptiveness_all(freqs):
    """Calculates relative codon adaptiveness, using all codons."""
    k = freqs.keys()
    v = freqs.values()
    n = norm_to_max(freqs.values())
    return dict(zip(freqs.keys(), norm_to_max(freqs.values())))

def codon_adaptiveness_blocks(freqs, blocks):
    """Calculates relative codon adaptiveness, using codon blocks."""
    result = freqs.copy()
    for b, codons in blocks.items():
        codon_vals = norm_to_max([result[c] for c in codons])
        for c, v in zip(codons, codon_vals):
            result[c] = v
    return result

def set_min(freqs, threshold):
    """Sets all values in freqs below min to specified threshold, in-place."""
    for k, v in freqs.items():
        if v < threshold:
            freqs[k] = threshold

def valid_codons(blocks):
    """Gets all valid codons from blocks"""
    result = []
    for b in blocks.values():
        result.extend(b)
    return result

def make_cai_1(genetic_code=SGC):
    """Returns function that calculates CAI model 1 using specified gen code."""
    blocks = get_synonyms(genetic_code)
    codons = frozenset(valid_codons(blocks))

    def cai_1(ref_freqs, gene_freqs, average=arithmetic_mean, \
        threshold=FREQUENCY_MIN):
        """ Assumes codon and amino acid selection to maximize translation rate.

        Gene length may also be under selection.

        ref_freqs: dict mapping reference set of codons for
            highly expressed genes to their frequencies of occurrence.
        gene_freqs: dictionary mapping codons to their usage in gene of
            interest.
        average: function for normalizing and averaging gene of interest codon
            frequencies.
        threshold: cutoff for small normalized codon frequencies to avoid bias.
        """
        r = ref_freqs.copy()
        set_min(r, threshold)
        adaptiveness_values = codon_adaptiveness_all(r)
        curr_codons = [k for k in gene_freqs if k in codons]
        return average([adaptiveness_values[i] for i in curr_codons],\
            [gene_freqs[i] for i in curr_codons])
    return cai_1

cai_1 = make_cai_1()

def make_cai_2(genetic_code=SGC):
    """Returns function that calculates CAI model 2 using specified gen code."""
    blocks = get_synonyms(genetic_code)
    codons = frozenset(valid_codons(blocks))

    def cai_2(ref_freqs, gene_freqs, average=arithmetic_mean, \
        threshold=FREQUENCY_MIN):
        """ Assumes codon, but not amino acid, selection to maximize translation
            rate (using geometric mean - Sharp and Li, 1987).

        ref_freqs: dict mapping reference set of codons for
            highly expressed genes to their frequencies of occurrence.
        gene_freqs: dictionary mapping codons to their usage in the gene of
            interest.
        average: function for normalizing and averaging the gene of interest codon
            frequencies.
        threshold: cutoff for small normalized codon frequencies to avoid bias.
        """
        r = ref_freqs.copy()
        set_min(r, threshold)
        adaptiveness_values = codon_adaptiveness_blocks(r, blocks)
        curr_codons = [k for k in gene_freqs if k in codons]
        return average([adaptiveness_values[i] for i in curr_codons],\
            [gene_freqs[i] for i in curr_codons])
    return cai_2

cai_2 = make_cai_2()

def make_cai_3(genetic_code=SGC):
    """Returns function that calculates CAI model 3 using specified gen code."""
    blocks = get_synonyms(genetic_code)

    def cai_3(ref_freqs, gene_freqs, average=arithmetic_mean, \
        threshold=FREQUENCY_MIN):
        """ Assumes codon, but not amino acid, selection to maximize translation
            rate, and per codon translation rate is paramount (using geometric 
            mean -- Eyre-Walker, 1996).

        ref_freqs: dict mapping reference set of codons for
            highly expressed genes to their frequencies of occurrence.

        gene_freqs: dictionary mapping codons to their usage in the gene of
            interest.

        average: function for normalizing and averaging the gene of interest 
            codon frequencies. if average is the string 'eyre-walker',
            will calculate CAI exactly according to the formula in his 1996 
            paper:

            CAI = exp(sum_i(sum_j(X_ij*ln_w_ij)/sum(X_ij))/m)
            ...where i is the codon block, j is the codon, X_ij is the freq,
            and w_ij is the relative adaptiveness of each codon.

            In other words, this version averages the log of the geometric
            means for each codon family, then exponentiates the result. Note
            that this actually produces exactly the same results as performing
            the geometric mean at both steps, so in general you'll just want to
            pass in the geometric mean ass the averaging function if you want to
            do this.

            Otherwise, if average is a function of the weights and frequencies,
            the same averaging function will be applied within and between
            families.

        threshold: cutoff for small normalized codon frequencies to avoid bias.
        """
        if average == 'eyre_walker':
            eyre_walker = True
            average = geometric_mean
        else:
            eyre_walker = False
        r = ref_freqs.copy()
        set_min(r, threshold)
        adaptiveness_values = codon_adaptiveness_blocks(r, blocks)
        block_results = []
        for b, codons in blocks.items():
            vals = [adaptiveness_values[i] for i in codons if i in gene_freqs]
            freqs = [gene_freqs[i] for i in codons if i in gene_freqs]
            #will skip if freqs missing
            if sum(freqs):
                block_results.append(average(vals, freqs))
        if eyre_walker:
            return exp(arithmetic_mean(log(block_results)))
        else:
            return average(block_results)
    return cai_3

cai_3 = make_cai_3()

#Define dict containing standard CAI variants
cais = {
    '1a' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_1(ref, gene, arithmetic_mean, threshold),
    '1g' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_1(ref, gene, geometric_mean, threshold),

    '2a' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_2(ref, gene, arithmetic_meani, threshold),
    '2g' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_2(ref, gene, geometric_mean, threshold),

    '3a' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_3(ref,gene, arithmetic_mean, threshold),
    '3g' : lambda ref, gene, threshold=FREQUENCY_MIN: \
        cai_3(ref,gene, geometric_mean, threshold),
}
